@@ -1,13 +1,10 @@
-/* $Id: data_thread.c 835 2007-09-19 18:53:14Z bnicholes $ */
+/* $Id: data_thread.c 1866 2008-10-17 15:49:38Z carenas $ */
 #include <stdio.h>
 #include <stdlib.h>
 #include <sys/poll.h>
 #include <sys/time.h>
 #include <gmetad.h>
 #include <string.h>
-#include <gmetad.h>
-
-extern int debug_level;
 
 extern hash_t *xml;
 
@@ -29,9 +26,9 @@ data_thread ( void *arg )
    struct pollfd struct_poll;
    struct timeval start, end;
  
-   if(debug_level)
+   if(get_debug_msg_level())
       {
-         fprintf(stderr,"Data thread %d is monitoring [%s] data source\n", (int) pthread_self(), d->name);
+         fprintf(stderr,"Data thread %lu is monitoring [%s] data source\n", (unsigned long)pthread_self(), d->name);
          for(i = 0; i < d->num_sources; i++)
             {
                addr = d->sources[i];
@@ -54,14 +51,14 @@ data_thread ( void *arg )
    for (;;)
       {
          gettimeofday(&start, NULL);
-	 sock = NULL;
-	 
-	 /* If we successfully read from a good data source last time then try the same host again first. */
-	 if(d->last_good_index >= 0)
-	   sock = g_tcp_socket_new ( d->sources[d->last_good_index] );
+         sock = NULL;
+         
+         /* If we successfully read from a good data source last time then try the same host again first. */
+         if(d->last_good_index != -1)
+           sock = g_tcp_socket_new ( d->sources[d->last_good_index] );
 
-	 /* If there was no good connection last time or the above connect failed then try each host in the list. */
-	 if(!sock)
+         /* If there was no good connection last time or the above connect failed then try each host in the list. */
+         if(!sock)
            {
              for(i=0; i < d->num_sources; i++)
                {
@@ -93,14 +90,22 @@ data_thread ( void *arg )
                if( rval < 0 )
                   {
                      /* Error */
-		    err_msg("poll() error in data_thread for [%s] data source after %d bytes read", d->name, read_index);
+                     err_msg("poll() error in data_thread from source %d for [%s] data source after %d bytes read", d->last_good_index, d->name, read_index);
+                     if (d->last_good_index < (d->num_sources - 1))
+                        d->last_good_index += 1; /* skip this source */
+                     else
+                        d->last_good_index = -1; /* forget this source */
                      d->dead = 1;
                      goto take_a_break;
                   }
                else if (rval == 0)
                   {
                      /* No revents during timeout period */
-                     err_msg("poll() timeout for [%s] data source after %d bytes read", d->name, read_index);
+                     err_msg("poll() timeout from source %d for [%s] data source after %d bytes read", d->last_good_index, d->name, read_index);
+                     if (d->last_good_index < (d->num_sources - 1))
+                        d->last_good_index += 1; /* skip this source */
+                     else
+                        d->last_good_index = -1; /* forget this source */
                      d->dead = 1;
                      goto take_a_break; 
                   }
@@ -122,6 +127,7 @@ data_thread ( void *arg )
                            if (bytes_read < 0)
                               {
                                  err_msg("data_thread() unable to read() socket for [%s] data source", d->name);
+                                 d->last_good_index = -1;
                                  d->dead = 1;
                                  goto take_a_break;
                               }
@@ -131,26 +137,29 @@ data_thread ( void *arg )
                               }
                            read_index+= bytes_read;
                         }
-       /* Appears that OSX uses POLLHUP on Sockets that I have loaded the entire message into the buffer...
-        * not that I lost the connection (See FreeBSD lists on this discussion)
-        */
-       #if !(defined(DARWIN))
+                        /* Appears that OSX uses POLLHUP on Sockets that I have loaded the entire message into the buffer...
+                         * not that I lost the connection (See FreeBSD lists on this discussion)
+                         */
+#if !(defined(DARWIN))
                      if( struct_poll.revents & POLLHUP )
                         {
                            err_msg("The remote machine closed connection for [%s] data source after %d bytes read", d->name, read_index);
+                           d->last_good_index = -1;
                            d->dead = 1;
                            goto take_a_break;
                         }
-       #endif /* DARWIN */
+#endif /* DARWIN */
                      if( struct_poll.revents & POLLERR )
                         {
                            err_msg("POLLERR! for [%s] data source after %d bytes read", d->name, read_index);
+                           d->last_good_index = -1;
                            d->dead = 1;
                            goto take_a_break;
                         }
                      if( struct_poll.revents & POLLNVAL )
                         {
                            err_msg("POLLNVAL! for [%s] data source after %d bytes read", d->name, read_index);
+                           d->last_good_index = -1;
                            d->dead = 1;
                            goto take_a_break;
                         }
@@ -175,10 +184,6 @@ data_thread ( void *arg )
        take_a_break:
          g_tcp_socket_delete(sock);
 
-	 /* Don't remember this host if there was a problem */
-	 if(d->dead)
-           d->last_good_index = -1;
-
          gettimeofday(&end, NULL);
          /* Sleep somewhere between (step +/- 5sec.) */
          sleep_time = (d->step - 5) + (10 * (rand()/(float)RAND_MAX)) - (end.tv_sec - start.tv_sec);
