@@ -1,4 +1,4 @@
-/* $Id: gmetad.c 681 2006-08-30 10:08:13Z knobi1 $ */
+/* $Id: gmetad.c 1233 2008-04-13 07:48:32Z carenas $ */
 #include <stdio.h>
 #include <stdlib.h>
 #include <signal.h>
@@ -12,6 +12,7 @@
 #include <cmdline.h>
 #include "daemon_init.h"
 
+#include "rrd_helpers.h"
 
 /* Holds our data sources. */
 hash_t *sources;
@@ -30,13 +31,11 @@ extern void* server_thread(void *);
 extern int parse_config_file ( char *config_file );
 extern int number_of_datasources ( char *config_file );
 extern struct type_tag* in_type_list (char *, unsigned int);
-extern int write_data_to_rrd( const char *source, const char *host, const char *metric,
-   const char *sum, const char *num, unsigned int step, unsigned int time_polled);
 
 struct gengetopt_args_info args_info;
 
 extern gmetad_config_t gmetad_config;
-extern int debug_level;
+static int debug_level;
 
 /* In cleanup.c */
 extern void *cleanup_thread(void *arg);
@@ -91,20 +90,20 @@ getfield(char* buf, short int index)
 int
 addstring(char *strings, int *edge, const char *s)
 {
-	int e = *edge;
-	int end = e + strlen(s) + 1;
+   int e = *edge;
+   int end = e + strlen(s) + 1;
 
-	/* I wish C had real exceptions. */
-	if (e > GMETAD_FRAMESIZE || end > GMETAD_FRAMESIZE)
-	{
-		err_msg("Field is too big!!");
-		return -1;
-	}
+   /* I wish C had real exceptions. */
+   if (e > GMETAD_FRAMESIZE || end > GMETAD_FRAMESIZE)
+   {
+      err_msg("Field is too big!!");
+      return -1;
+   }
 
-	strcpy(strings + e, s);
-	*edge = end;
+   strcpy(strings + e, s);
+   *edge = end;
 
-	return e;
+   return e;
 }
 
 
@@ -149,14 +148,13 @@ sum_metrics(datum_t *key, datum_t *val, void *arg)
          tt = in_type_list(type, strlen(type));
          if (!tt) return 0;
 
+         /* We sum everything in double to properly combine integer sources
+            (3.0) with float sources (3.1).  This also avoids wraparound
+            errors: for example memory KB exceeding 4TB. */
          switch (tt->type)
             {
                case INT:
-                  rootmetric->val.int32 += metric->val.int32;
-                  break;
                case UINT:
-                  rootmetric->val.uint32 += metric->val.uint32;
-                  break;
                case FLOAT:
                   rootmetric->val.d += metric->val.d;
                   break;
@@ -191,8 +189,6 @@ do_root_summary( datum_t *key, datum_t *val, void *arg )
    /* Need to be sure the source has a complete sum for its metrics. */
    pthread_mutex_lock(source->sum_finished);
 
-    /* err_msg("Doing root summary for source %s", source->ds->name); */
-
    /* We know that all these metrics are numeric. */
    rc = hash_foreach(source->metric_summary, sum_metrics, arg);
 
@@ -200,6 +196,7 @@ do_root_summary( datum_t *key, datum_t *val, void *arg )
    root.hosts_up += source->hosts_up;
    root.hosts_down += source->hosts_down;
 
+   /* summary completed for source */
    pthread_mutex_unlock(source->sum_finished);
 
    return rc;
@@ -222,29 +219,20 @@ write_root_summary(datum_t *key, datum_t *val, void *arg)
 
    /* Summarize all numeric metrics */
    tt = in_type_list(type, strlen(type));
-   if (!tt) return 0;
+   /* Don't write a summary for an unknown or STRING type. */
+   if (!tt || (tt->type == STRING)) 
+       return 0;
 
-   switch (tt->type)
-      {
-         case INT:
-            sprintf(sum, "%d", metric->val.int32);
-            break;
-         case UINT:
-            sprintf(sum, "%u", metric->val.uint32);
-            break;
-         case FLOAT:
-            sprintf(sum, "%.5f", metric->val.d);
-            break;
-         default:
-            break;
-      }
+   /* We log all our sums in double which does not suffer from
+      wraparound errors: for example memory KB exceeding 4TB. -twitham */
+   sprintf(sum, "%.5f", metric->val.d);
 
    sprintf(num, "%u", metric->num);
 
    /* err_msg("Writing Overall Summary for metric %s (%s)", name, sum); */
 
    /* Save the data to a round robin database */
-   rc = write_data_to_rrd( NULL, NULL, name, sum, num, 15, 0);
+   rc = write_data_to_rrd( NULL, NULL, name, sum, num, 15, 0, metric->slope);
    if (rc)
       {
          err_msg("Unable to write meta data for metric %s to RRD", name);
@@ -310,6 +298,7 @@ main ( int argc, char *argv[] )
          c->debug_level = args_info.debug_arg;
       }
    debug_level = c->debug_level;
+   set_debug_msg_level(debug_level);
 
    /* Setup our default authority pointer if the conf file hasnt yet.
     * Done in the style of hash node strings. */
@@ -393,6 +382,11 @@ main ( int argc, char *argv[] )
       }
    debug_msg("interactive xml listening on port %d", c->interactive_port);
 
+   /* initialize summary mutex */
+   root.sum_finished = (pthread_mutex_t *) 
+                          malloc(sizeof(pthread_mutex_t));
+   pthread_mutex_init(root.sum_finished, NULL);
+
    pthread_attr_init( &attr );
    pthread_attr_setdetachstate( &attr, PTHREAD_CREATE_DETACHED );
 
@@ -417,6 +411,9 @@ main ( int argc, char *argv[] )
          sleep_time = 10 + ((30-10)*1.0) * rand()/(RAND_MAX + 1.0);
          sleep(sleep_time);
 
+         /* Need to be sure root is locked while doing summary */
+         pthread_mutex_lock(root.sum_finished);
+
          /* Flush the old values */
          hash_foreach(root.metric_summary, zero_out_summary, NULL);
          root.hosts_up = 0;
@@ -425,10 +422,12 @@ main ( int argc, char *argv[] )
          /* Sum the new values */
          hash_foreach(root.authority, do_root_summary, NULL );
 
+         /* summary completed */
+         pthread_mutex_unlock(root.sum_finished);
+
          /* Save them to RRD */
          hash_foreach(root.metric_summary, write_root_summary, NULL);
       }
 
    return 0;
 }
-
