@@ -5,10 +5,9 @@
 #include "config.h"
 #endif
 
+#include "ganglia_priv.h"
 #include "net.h"
 #include "hash.h"
-#include "debug_msg.h"
-#include "error.h"
 #include "llist.h"
 #include "conf.h"
 #include "become_a_nobody.h"
@@ -68,7 +67,9 @@ typedef enum
       DOWN_TAG,
       METRICS_TAG,
       SUM_TAG,
-      NUM_TAG
+      NUM_TAG,
+      EXTRA_DATA_TAG,
+      EXTRA_ELEMENT_TAG
    }
 xml_tag_t;
 
@@ -106,7 +107,7 @@ typedef struct
       int dead;
       int last_good_index;
    }
-data_source_list_t;;
+data_source_list_t;
 
 /* The size of an ethernet frame, minus IP/UDP headers (1472)
  * plus a bit since gmond sends meta data in binary format, while
@@ -118,8 +119,6 @@ data_source_list_t;;
  * metric_t to compute summaries. */
 typedef union
    {
-      int32_t int32;
-      uint32_t uint32;
       double d;
       int str;
    }
@@ -202,6 +201,8 @@ typedef struct
 Host_t;
 
 
+#define MAX_EXTRA_ELEMENTS  32
+
 /* sacerdoti: Since we don't know the length of the string fields,
  * we place them sequentially in the strings buffer. The
  * order of strings in the buffer is usually:
@@ -216,6 +217,7 @@ typedef struct
       hash_t *leaf;  /* Always NULL. */
       struct timeval t0;
       metric_val_t val;
+      short int name;
       short int valstr;    /* An optimization to speed queries. */
       short int precision;    /* Number of decimal places for floats. */
       uint32_t num;
@@ -226,6 +228,10 @@ typedef struct
       uint32_t dmax;
       short int slope;
       short int source;
+      short int ednameslen;
+      short int edvalueslen;
+      short int ednames[MAX_EXTRA_ELEMENTS];
+      short int edvalues[MAX_EXTRA_ELEMENTS];
       short int stringslen;
       char strings[GMETAD_FRAMESIZE];
    }
