@@ -1,16 +1,12 @@
-/* $Id: process_xml.c 670 2006-07-21 08:59:40Z knobi1 $ */
+/* $Id: process_xml.c 1656 2008-08-11 21:02:36Z bnicholes $ */
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <sys/time.h>
-#include "expat.h"
+#include <expat.h>
 #include "gmetad.h"
+#include "rrd_helpers.h"
 
-
-
-extern int write_data_to_rrd( const char *source, const char *host, 
-                const char *metric, const char *sum, const char *num, 
-                unsigned int step, unsigned int time_polled);
 extern int zero_out_summary(datum_t *key, datum_t *val, void *arg);
 extern char* getfield(char *buf, short int index);
 
@@ -103,16 +99,10 @@ fillmetric(const char** attr, Metric_t *metric, const char* type)
                   switch (tt->type)
                      {
                         case INT:
-                           metric->val.int32 = (long) 
-                                   strtol(metricval, (char**) NULL, 10);
-                           break;
                         case TIMESTAMP:
                         case UINT:
-                           metric->val.uint32 = (unsigned long) 
-                                   strtoul(metricval, (char**) NULL, 10);
-                           break;
                         case FLOAT:
-                           metric->val.d = (double) 
+                           metric->val.d = (double)
                                    strtod(metricval, (char**) NULL);
                            p = strrchr(metricval, '.');
                            if (p) metric->precision = (short int) strlen(p+1);
@@ -216,7 +206,7 @@ startElement_GRID(void *data, const char *el, const char **attr)
                source->ds = xmldata->ds;
 
                /* Initialize the partial sum lock */
-               source->sum_finished = (pthread_mutex_t*) 
+               source->sum_finished = (pthread_mutex_t *) 
                        malloc(sizeof(pthread_mutex_t));
                pthread_mutex_init(source->sum_finished, NULL);
 
@@ -228,13 +218,14 @@ startElement_GRID(void *data, const char *el, const char **attr)
             {  /* Found Cluster. Put into our Source buffer in xmldata. */
                memcpy(source, hash_datum->data, hash_datum->size);
                datum_free(hash_datum);
-               source->hosts_up = 0;
-               source->hosts_down = 0;
 
                /* Grab the "partial sum" mutex until we are finished
                 * summarizing. Needs to be done asap.*/
                pthread_mutex_lock(source->sum_finished);
 
+               source->hosts_up = 0;
+               source->hosts_down = 0;
+
                hash_foreach(source->metric_summary, zero_out_summary, NULL);
             }
 
@@ -332,7 +323,7 @@ startElement_CLUSTER(void *data, const char *el, const char **attr)
          source->ds = xmldata->ds;
          
          /* Initialize the partial sum lock */
-         source->sum_finished = (pthread_mutex_t*) 
+         source->sum_finished = (pthread_mutex_t *) 
                  malloc(sizeof(pthread_mutex_t));
          pthread_mutex_init(source->sum_finished, NULL);
 
@@ -344,12 +335,12 @@ startElement_CLUSTER(void *data, const char *el, const char **attr)
          memcpy(source, hash_datum->data, hash_datum->size);
          datum_free(hash_datum);
 
-         source->hosts_up = 0;
-         source->hosts_down = 0;
-
          /* We need this lock before zeroing metric sums. */
          pthread_mutex_lock(source->sum_finished);
 
+         source->hosts_up = 0;
+         source->hosts_down = 0;
+
          hash_foreach(source->metric_summary, zero_out_summary, NULL);
       }
 
@@ -568,6 +559,7 @@ static int
 startElement_METRIC(void *data, const char *el, const char **attr)
 {
    xmldata_t *xmldata = (xmldata_t *)data;
+   ganglia_slope_t slope = GANGLIA_SLOPE_UNSPECIFIED;
    struct xml_tag *xt;
    struct type_tag *tt;
    datum_t *hash_datum = NULL;
@@ -577,7 +569,7 @@ startElement_METRIC(void *data, const char *el, const char **attr)
    const char *metricval = NULL;
    const char *type = NULL;
    int do_summary;
-   int i;
+   int i, edge;
    hash_t *summary;
    Metric_t *metric;
 
@@ -602,6 +594,8 @@ startElement_METRIC(void *data, const char *el, const char **attr)
                case TYPE_TAG:
                   type = attr[i+1];
                   break;
+               case SLOPE_TAG:
+                  slope = cstr_to_slope(attr[i+1]);
                default:
                   break;
             }
@@ -629,13 +623,16 @@ startElement_METRIC(void *data, const char *el, const char **attr)
                                   xmldata->hostname, name);
                   xmldata->rval = write_data_to_rrd(xmldata->sourcename,
                         xmldata->hostname, name, metricval, NULL,
-                        xmldata->ds->step, xmldata->source.localtime);
+                        xmldata->ds->step, xmldata->source.localtime, slope);
             }
          metric->id = METRIC_NODE;
          metric->report_start = metric_report_start;
          metric->report_end = metric_report_end;
 
          fillmetric(attr, metric, type);
+         edge = metric->stringslen;
+         metric->name = addstring(metric->strings, &edge, name);
+         metric->stringslen = edge;
 
          /* Set local idea of T0. */
          metric->t0 = xmldata->now;
@@ -677,15 +674,9 @@ startElement_METRIC(void *data, const char *el, const char **attr)
                switch (tt->type)
                   {
                      case INT:
-                        metric->val.int32 += (long) 
-                                strtol(metricval, (char**) NULL, 10);
-                        break;
                      case UINT:
-                        metric->val.uint32 += (unsigned long) 
-                                strtoul(metricval, (char**) NULL, 10);
-                        break;
                      case FLOAT:
-                        metric->val.d += (double) 
+                        metric->val.d += (double)
                                 strtod(metricval, (char**) NULL);
                         break;
                      default:
@@ -708,6 +699,147 @@ startElement_METRIC(void *data, const char *el, const char **attr)
 }
 
 
+static int
+startElement_EXTRA_DATA(void *data, const char *el, const char **attr)
+{
+   return 0;
+}
+
+/* XXX - There is an issue which will cause a failure if there are more than 16
+  EXTRA_ELEMENTs.  This is a problem with the size of the data structure that
+  is used to hold the metric information.
+*/
+static int
+startElement_EXTRA_ELEMENT (void *data, const char *el, const char **attr)
+{
+    xmldata_t *xmldata = (xmldata_t *)data;
+    int edge;
+    struct xml_tag *xt;
+    int i, name_off, value_off;
+    Metric_t metric;
+    char *name = getfield(xmldata->metric.strings, xmldata->metric.name);
+    datum_t *rdatum;
+    datum_t hashkey, hashval;
+    datum_t *hash_datum = NULL;
+    
+    if (!xmldata->host_alive) 
+        return 0;
+
+    /* Only keep extra element details if we are the authority on this cluster. */
+    if (!authority_mode(xmldata))
+       return 0;
+
+    hashkey.data = (void*) name;
+    hashkey.size =  strlen(name) + 1;
+    
+    hash_datum = hash_lookup (&hashkey, xmldata->host.metrics);
+    if (!hash_datum) 
+        return 0;
+    
+    memcpy(&metric, hash_datum->data, hash_datum->size);
+    datum_free(hash_datum);
+
+    /* Check to make sure that we don't try to add more
+        extra elements than the array can handle.
+    */  
+    if (metric.ednameslen >= MAX_EXTRA_ELEMENTS) 
+    {
+        debug_msg("Can not add more extra elements for [%s].  Capacity of %d reached[%s].",
+                  name, MAX_EXTRA_ELEMENTS);
+        return 0;
+    }
+
+    edge = metric.stringslen;
+    
+    name_off = value_off = -1;
+    for(i = 0; attr[i]; i+=2)
+    {
+        xt = in_xml_list(attr[i], strlen(attr[i]));
+        if (!xt) 
+            continue;
+        switch (xt->tag)
+        {
+        case NAME_TAG:
+            name_off = i;
+            break;
+        case VAL_TAG:
+            value_off = i;
+            break;
+        default:
+            break;
+        }
+    }
+    
+    if ((name_off >= 0) && (value_off >= 0)) 
+    {
+        const char *new_name = attr[name_off+1];
+        const char *new_value = attr[value_off+1];
+
+        metric.ednames[metric.ednameslen++] = addstring(metric.strings, &edge, new_name);
+        metric.edvalues[metric.edvalueslen++] = addstring(metric.strings, &edge, new_value);
+    
+        metric.stringslen = edge;
+    
+        hashkey.data = (void*)name;
+        hashkey.size =  strlen(name) + 1;
+    
+        /* Trim metric structure to the correct length. */
+        hashval.size = sizeof(metric) - GMETAD_FRAMESIZE + metric.stringslen;
+        hashval.data = (void*) &metric;
+    
+        /* Update full metric in cluster host table. */
+        rdatum = hash_insert(&hashkey, &hashval, xmldata->host.metrics);
+        if (!rdatum)
+        {
+            err_msg("Could not insert %s metric", name);
+        }
+        else
+        {
+            hash_t *summary = xmldata->source.metric_summary;
+            Metric_t sum_metric;
+
+            /* only update summary if metric is in hash */
+            hash_datum = hash_lookup(&hashkey, summary);
+
+            if (hash_datum) {
+                int found = FALSE;
+
+                memcpy(&sum_metric, hash_datum->data, hash_datum->size);
+                datum_free(hash_datum);
+
+                for (i = 0; i < sum_metric.ednameslen; i++) {
+                    char *chk_name = getfield(sum_metric.strings, sum_metric.ednames[i]);
+                    char *chk_value = getfield(sum_metric.strings, sum_metric.edvalues[i]);
+
+                    /* If the name and value already exists, skip adding the strings. */
+                    if (!strcasecmp(chk_name, new_name) && !strcasecmp(chk_value, new_value)) {
+                        found = TRUE;
+                        break;
+                    }
+                }
+                if (!found) {
+                    edge = sum_metric.stringslen;
+                    sum_metric.ednames[sum_metric.ednameslen++] = addstring(sum_metric.strings, &edge, new_name);
+                    sum_metric.edvalues[sum_metric.edvalueslen++] = addstring(sum_metric.strings, &edge, new_value);
+                    sum_metric.stringslen = edge;
+                }
+
+                /* Trim graph display sum_metric at (352, 208) now or when in startElement_EXTRA_ELEMENT
+metric structure to the correct length. Tricky. */
+                hashval.size = sizeof(sum_metric) - GMETAD_FRAMESIZE + sum_metric.stringslen;
+                hashval.data = (void*) &sum_metric;
+
+                /* Update metric in summary table. */
+                rdatum = hash_insert(&hashkey, &hashval, summary);
+                if (!rdatum)
+                    err_msg("Could not insert summary %s metric", name);
+            }
+        }
+    }
+    
+    return 0;
+}
+
 static int
 startElement_METRICS(void *data, const char *el, const char **attr)
 {
@@ -773,15 +905,9 @@ startElement_METRICS(void *data, const char *el, const char **attr)
          switch (tt->type)
                {
                   case INT:
-                     metric->val.int32 += (long) 
-                             strtol(metricval, (char**) NULL, 10);
-                     break;
                   case UINT:
-                     metric->val.uint32 += (unsigned long) 
-                             strtoul(metricval, (char**) NULL, 10);
-                     break;
                   case FLOAT:
-                     metric->val.d += (double) 
+                     metric->val.d += (double)
                              strtod(metricval, (char**) NULL);
                      break;
                   default:
@@ -880,6 +1006,14 @@ start (void *data, const char *el, const char **attr)
             rc = startElement_GANGLIA_XML(data, el, attr);
             break;
 
+         case EXTRA_DATA_TAG:
+            rc = startElement_EXTRA_DATA(data, el, attr);
+            break;
+
+         case EXTRA_ELEMENT_TAG:
+            rc = startElement_EXTRA_ELEMENT(data, el, attr);
+            break;
+    
          default:
             break;
       }  /* end switch */
@@ -914,10 +1048,8 @@ finish_processing_source(datum_t *key, datum_t *val, void *arg)
    switch (tt->type)
       {
          case INT:
-            sprintf(sum, "%d", metric->val.int32);
-            break;
          case UINT:
-            sprintf(sum, "%u", metric->val.uint32);
+            sprintf(sum, "%.f", metric->val.d);
             break;
          case FLOAT:
             sprintf(sum, "%.*f", (int) metric->precision, metric->val.d);
@@ -929,14 +1061,17 @@ finish_processing_source(datum_t *key, datum_t *val, void *arg)
 
    /* Save the data to a round robin database if this data source is 
     * alive. */
-   if (!xmldata->ds->dead && !xmldata->rval) {
+   if (!xmldata->ds->dead && !xmldata->rval) 
+     {
    
-	   debug_msg("Writing Summary data for source %s, metric %s",
-	       xmldata->sourcename, name);
-	
-	   xmldata->rval = write_data_to_rrd(xmldata->sourcename, NULL, name,
-	           sum, num, xmldata->ds->step, xmldata->source.localtime);
-   }
+       debug_msg("Writing Summary data for source %s, metric %s",
+                 xmldata->sourcename, name);
+
+       xmldata->rval = write_data_to_rrd(xmldata->sourcename, NULL, name,
+                                         sum, num, xmldata->ds->step,
+                                         xmldata->source.localtime,
+                                         cstr_to_slope(getfield(metric->strings, metric->slope)));
+     }
 
    return xmldata->rval;
 }
@@ -946,6 +1081,10 @@ static int
 endElement_GRID(void *data, const char *el)
 {
    xmldata_t *xmldata = (xmldata_t *) data;
+   Source_t *source = &xmldata->source;
+
+   /*release the partial sum mutex */
+   pthread_mutex_unlock(source->sum_finished);
 
    if (gmetad_config.scalable_mode)
       {
@@ -973,7 +1112,6 @@ endElement_CLUSTER(void *data, const char *el)
 
          /* Release the partial sum mutex */
          pthread_mutex_unlock(source->sum_finished);
-         /*err_msg("%s releasing lock", xmldata->sourcename);*/
 
          hashkey.data = (void*) xmldata->sourcename;
          hashkey.size = strlen(xmldata->sourcename) + 1;
@@ -984,7 +1122,6 @@ endElement_CLUSTER(void *data, const char *el)
 
          /* We insert here to get an accurate hosts up/down value. */
          rdatum = hash_insert( &hashkey, &hashval, xmldata->root);
-         source->sum_finished = NULL; /* remember that we released the lock */
          if (!rdatum) {
                err_msg("Could not insert source %s", xmldata->sourcename);
                return 1;
@@ -1060,14 +1197,6 @@ process_xml(data_source_list_t *d, char *buf)
          xmldata.rval = 1;
       }
 
-   /* Release lock again for good measure (required under certain errors). */
-   if (xmldata.source.sum_finished)
-      {
-         rval = pthread_mutex_unlock(xmldata.source.sum_finished);
-         if (rval!=0)
-            err_msg("Could not release summary lock for %s", d->name);
-      }
-
    /* Free memory that might have been allocated in xmldata */
    if (xmldata.sourcename)
       free(xmldata.sourcename);
