@@ -1,7 +1,7 @@
-/* $Id: g25_config.c 590 2005-10-10 08:07:07Z knobi1 $ */
+/* $Id: g25_config.c 1899 2008-11-04 08:15:52Z carenas $ */
 #include <stdio.h>
 #include <dotconf.h>
-#include "ganglia.h"
+#include "ganglia_priv.h"
 #include "g25_config.h"
 
 gmond_config_t gmond_config;
@@ -128,7 +128,7 @@ static DOTCONF_CB(cb_trusted_hosts)
    
    for (i = 0; i < cmd->arg_count; i++)
       {
-	c->trusted_hosts[i] = strdup(cmd->data.list[i]);
+         c->trusted_hosts[i] = strdup(cmd->data.list[i]);
       }
    c->trusted_hosts[i] = NULL;
    return NULL;
@@ -162,6 +162,13 @@ static DOTCONF_CB(cb_deaf)
    return NULL;
 }
 
+static DOTCONF_CB(cb_allow_extra_data)
+{
+   gmond_config_t *c = (gmond_config_t *)cmd->option->info;
+   c->allow_extra_data = cmd->data.value;
+   return NULL;
+}
+
 static DOTCONF_CB(cb_debug_level)
 {
    gmond_config_t *c = (gmond_config_t *)cmd->option->info;
@@ -226,6 +233,7 @@ set_defaults(gmond_config_t *config )
    config->num_custom_metrics = 16;
    config->mute = 0;
    config->deaf = 0;
+   config->allow_extra_data = 1;
    config->debug_level = 0;
    config->no_setuid = 0;
    config->setuid = conf_strdup("nobody");
@@ -243,26 +251,27 @@ print_config(char *path, gmond_config_t *config)
   char *p;
 
   fprintf(stdout,"/* global variables */\n");
-  fprintf(stdout,"globals {\n  mute = \"%s\"\n  deaf = \"%s\"\n  debug_level = \"%ld\"\n  setuid = \"%s\"\n  user=\"%s\"\n  gexec = \"%s\"\n  host_dmax = \"%ld\"\n}\n\n",
-	  config->mute? "yes":"no", 
-	  config->deaf? "yes":"no", 
-	  config->debug_level, 
-	  config->no_setuid? "no":"yes",
-	  config->setuid,
-	  config->no_gexec? "no":"yes",
-	  config->host_dmax);
+  fprintf(stdout,"globals {\n  mute = \"%s\"\n  deaf = \"%s\"\n  allow_extra_data = \"%s\"\n  debug_level = \"%ld\"\n  setuid = \"%s\"\n  user=\"%s\"\n  gexec = \"%s\"\n  host_dmax = \"%ld\"\n}\n\n",
+          config->mute? "yes":"no", 
+          config->deaf? "yes":"no", 
+          config->allow_extra_data? "yes":"no", 
+          config->debug_level, 
+          config->no_setuid? "no":"yes",
+          config->setuid,
+          config->no_gexec? "no":"yes",
+          config->host_dmax);
 
   fprintf(stdout,"/* info about cluster  */\n");
   fprintf(stdout,"cluster {\n  name = \"%s\"\n  owner = \"%s\"\n  latlong = \"%s\"\n  url=\"%s\"\n}\n\n",
-	  config->name, config->owner, config->latlong, config->url);
+          config->name, config->owner, config->latlong, config->url);
 
   fprintf(stdout,"/* info about host */\n");
   fprintf(stdout,"host {\n  location = \"%s\"\n}\n\n",
-	  config->location);
+          config->location);
 
   fprintf(stdout,"/* channel to send multicast on mcast_channel:mcast_port */\n");
   fprintf(stdout,"udp_send_channel {\n  mcast_join = \"%s\"\n  port = \"%hd\"\n  ttl=\"%ld\"\n",
-	  config->mcast_channel, config->mcast_port, config->mcast_ttl);
+          config->mcast_channel, config->mcast_port, config->mcast_ttl);
   if(config->mcast_if_given)
     {
       fprintf(stdout,"  mcast_if = \"%s\"\n", config->mcast_if);
@@ -271,7 +280,7 @@ print_config(char *path, gmond_config_t *config)
 
   fprintf(stdout,"/* channel to receive multicast from mcast_channel:mcast_port */\n");
   fprintf(stdout,"udp_recv_channel {\n  mcast_join = \"%s\"\n  port = \"%hd\"\n  bind = \"%s\"\n",
-	  config->mcast_channel, config->mcast_port, config->mcast_channel);
+          config->mcast_channel, config->mcast_port, config->mcast_channel);
   if(config->mcast_if_given)
     {
       fprintf(stdout,"  mcast_if = \"%s\"\n", config->mcast_if);
@@ -283,12 +292,17 @@ print_config(char *path, gmond_config_t *config)
   if( config->trusted_hosts  && !config->all_trusted )
     {
       fprintf(stdout,"/* your trusted_hosts assuming ipv4 mask*/\n");
-      fprintf(stdout,"acl{\n  default=\"deny\"\n");
+      fprintf(stdout,"  acl {\n    default=\"deny\"\n");
       for(i = 0, p = config->trusted_hosts[i]; p; i++, p = config->trusted_hosts[i])
-	{
-	  fprintf(stdout, "  access {\n  ip=\"%s\"\n  mask = 32\n  action = \"allow\"\n  }\n", p);
-	}
-      fprintf(stdout,"}\n");/* close acl */
+        {
+          fprintf(stdout, "    access {\n");
+          fprintf(stdout, "      ip=\"%s\"\n      mask = 32\n      action = \"allow\"\n", p);
+          fprintf(stdout, "    }\n");
+        }
+      fprintf(stdout, "    access {\n");
+      fprintf(stdout, "      ip=\"127.0.0.1\"\n      mask = 32\n      action = \"allow\"\n");
+      fprintf(stdout, "    }\n");
+      fprintf(stdout,"  }\n");/* close acl */
     }
   fprintf(stdout,"}\n\n"); /* close tcp_accept_channel */
   fprintf(stdout,"%s\n", Ganglia_default_collection_groups());
@@ -320,6 +334,7 @@ print_ganglia_25_config( char *path )
          {"num_custom_metrics", ARG_INT, cb_num_custom_metrics, &gmond_config, 0},
          {"mute", ARG_TOGGLE, cb_mute, &gmond_config, 0},
          {"deaf", ARG_TOGGLE, cb_deaf, &gmond_config, 0},
+         {"allow_extra_data", ARG_TOGGLE, cb_allow_extra_data, &gmond_config, 0},
          {"debug_level", ARG_INT, cb_debug_level, &gmond_config, 0},
          {"no_setuid", ARG_TOGGLE, cb_no_setuid, &gmond_config, 0},
          {"setuid", ARG_STR, cb_setuid, &gmond_config, 0},
