@@ -4,7 +4,7 @@
 #include <string.h>
 
 /* autoconf me later */
-#define DEFAULT_GMOND_CONFIG_FILE "/etc/gmond.conf"
+#define DEFAULT_GMOND_CONFIG_FILE "/etc/ganglia/gmond.conf"
 
 typedef struct
    {
@@ -26,6 +26,7 @@ typedef struct
       long int num_custom_metrics;
       long int mute;
       long int deaf;
+      long int allow_extra_data;
       long int debug_level;
       long int no_setuid;
       char *setuid;
