@@ -5,18 +5,41 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <apr_strings.h>
-#include "apr_network_io.h"
-#include "apr_arch_networkio.h"
+#include <apr_network_io.h>
+#include <apr_portable.h>
 
-#include "apr_net.h"
+/* 
+ * FIXME: apr_arch_networkio.h is not public
+ *        define interfaces locally so they can be used
+ */
+void apr_sockaddr_vars_set(apr_sockaddr_t *, int, apr_port_t);
+
+/*
+ * FIXME: incomplete definition but enough to manipulate socketdes directly
+ */
+struct apr_socket_t {
+    apr_pool_t *pool;
+    int socketdes;
+    int type;
+    int protocol;
+};
+
+#include <apr_net.h>
+#include <apr_version.h>
 
 #include <sys/ioctl.h>
 #include <net/if.h>
 
-#ifdef SOLARIS2
+#ifdef SOLARIS
 #include <sys/sockio.h>  /* for SIOCGIFADDR */
+#include <unistd.h>      /* for ioctl */
+#include <stropts.h>
 #endif
 
+#include <gm_msg.h>
+
+const char *apr_inet_ntop(int af, const void *src, char *dst, apr_size_t size);
+
 /* This function is copied directly from the 
  * apr_sockaddr_ip_get() function and modified to take a static
  * buffer instead of needing to malloc memory from a pool */
@@ -45,10 +68,10 @@ APR_DECLARE(apr_status_t) apr_sockaddr_ip_buffer_get(char *addr, int len,
 
         /* use memmove since the memory areas overlap */
         memmove( addr, addr+7, strlen(addr+7) + 1);/* +1 for \0 */
-	
-	/* old code
+        
+        /* old code
         *addr += strlen("::ffff:");
-	*/
+        */
     }
 #endif
     return APR_SUCCESS;
@@ -70,14 +93,14 @@ create_net_client(apr_pool_t *context, int type, char *host, apr_port_t port)
   family = remotesa->sa.sin.sin_family;
 
   /* Created the socket */
-  status = apr_socket_create(&sock, family, type, context);
+  status = apr_socket_create(&sock, family, type, 0, context);
   if(status != APR_SUCCESS)
     {
       return NULL;
     }
 
   /* Connect the socket to the address */
-  status = apr_connect(sock, remotesa);
+  status = apr_socket_connect(sock, remotesa);
   if(status != APR_SUCCESS)
     {
       apr_socket_close(sock);
@@ -94,8 +117,8 @@ create_udp_client(apr_pool_t *context, char *host, apr_port_t port)
 }
 
 static apr_socket_t *
-create_net_server(apr_pool_t *context, int32_t ofamily, int type, apr_port_t
-		port, char *bind_addr, int blocking)
+create_net_server(apr_pool_t *context, int32_t ofamily, int type, apr_port_t port, 
+                  char *bind_addr, int blocking)
 {
   apr_sockaddr_t *localsa = NULL;
   apr_socket_t *sock = NULL;
@@ -115,13 +138,13 @@ create_net_server(apr_pool_t *context, int32_t ofamily, int type, apr_port_t
       family = localsa->sa.sin.sin_family;
     }
 
-  stat = apr_socket_create(&sock, family, type, context);
+  stat = apr_socket_create(&sock, family, type, 0, context);
   if( stat != APR_SUCCESS )
     return NULL;
 
   if(!blocking){
      /* This is a non-blocking server */
-     stat = apr_setsocketopt(sock, APR_SO_NONBLOCK, 1);
+     stat = apr_socket_opt_set(sock, APR_SO_NONBLOCK, 1);
      if (stat != APR_SUCCESS)
      {
            apr_socket_close(sock);
@@ -129,7 +152,7 @@ create_net_server(apr_pool_t *context, int32_t ofamily, int type, apr_port_t
      }
   }
 
-  stat = apr_setsocketopt(sock, APR_SO_REUSEADDR, 1);
+  stat = apr_socket_opt_set(sock, APR_SO_REUSEADDR, 1);
   if (stat != APR_SUCCESS)
     {
       apr_socket_close(sock);
@@ -139,7 +162,7 @@ create_net_server(apr_pool_t *context, int32_t ofamily, int type, apr_port_t
   if(!localsa)
     {
       apr_socket_addr_get(&localsa, APR_LOCAL, sock);
-      apr_sockaddr_port_set(localsa, port);
+      apr_sockaddr_vars_set(localsa, localsa->family, port);
     }
 
 #if APR_HAVE_IPV6
@@ -149,16 +172,16 @@ create_net_server(apr_pool_t *context, int32_t ofamily, int type, apr_port_t
        /* Don't accept IPv4 connections on an IPv6 listening socket */
        stat = apr_socket_opt_set(sock, APR_IPV6_V6ONLY, one);
        if(stat == APR_ENOTIMPL)
-	 {
-	   fprintf(stderr,"Warning: your operating system does not support IPV6_V6ONLY!\n");
-	   fprintf(stderr,"This means that you are also listening to IPv4 traffic on port %d\n",
-		   port);
-	   fprintf(stderr,"This IPv6=>IPv4 mapping may be a security risk.\n");
+         {
+           err_msg("Warning: your operating system does not support IPV6_V6ONLY!\n");
+           err_msg("This means that you are also listening to IPv4 traffic on port %d\n",
+               port);
+           err_msg("This IPv6=>IPv4 mapping may be a security risk.\n");
          }
      }
 #endif
 
-  stat = apr_bind(sock, localsa);
+  stat = apr_socket_bind(sock, localsa);
   if( stat != APR_SUCCESS)
     {
        apr_socket_close(sock);
@@ -169,67 +192,32 @@ create_net_server(apr_pool_t *context, int32_t ofamily, int type, apr_port_t
 }
 
 apr_socket_t *
-create_udp_server(apr_pool_t *context, int32_t family, apr_port_t port, char
-		*bind_addr)
+create_udp_server(apr_pool_t *context, int32_t family, apr_port_t port, 
+                  char *bind_addr)
 {
   return create_net_server(context, family, SOCK_DGRAM, port, bind_addr, 0);
 }
 
 apr_socket_t *
-create_tcp_server(apr_pool_t *context, int32_t family, apr_port_t port, char
-		*bind_addr, char *interface, int blocking)
+create_tcp_server(apr_pool_t *context, int32_t family, apr_port_t port, 
+                  char *bind_addr, char *interface, int blocking)
 {
   apr_socket_t *sock = create_net_server(context, family, SOCK_STREAM, port,
-		  bind_addr, blocking);
+                                         bind_addr, blocking);
   if(!sock)
     {
       return NULL;
     }
-  if(apr_listen(sock, 5) != APR_SUCCESS) 
+  if(apr_socket_listen(sock, 5) != APR_SUCCESS) 
     {
       return NULL;
     }
   return sock;
 }
 
-int
-mcast_set_ttl(apr_socket_t *socket, int val)
-{
-  apr_sockaddr_t *sa;
+/*XXX This should really be replaced by the APR mcast functions */
 
-  apr_socket_addr_get(&sa, APR_LOCAL, socket);
-  if(!sa)
-    {
-      return -1;
-    }
-  switch (sa->family)
-    {
-	case APR_INET: {
-		u_char		ttl;
-
-		ttl = val;
-		return(setsockopt(socket->socketdes, IPPROTO_IP, IP_MULTICAST_TTL,
-						  &ttl, sizeof(ttl)));
-	}
-
-#if	APR_HAVE_IPV6
-	case APR_INET6: {
-		int		hop;
-
-		hop = val;
-		return(setsockopt(socket->socketdes, IPPROTO_IPV6, IPV6_MULTICAST_HOPS,
-						  &hop, sizeof(hop)));
-	}
-#endif
-
-	default:
-		errno = EPROTONOSUPPORT;
-		return(-1);
-	}
-}
-
-
-/**
+/*
  *  Configure from which interface multicast traffic should be sent.
  */
 static apr_status_t
@@ -238,78 +226,74 @@ mcast_emit_on_if( apr_pool_t *context, apr_socket_t *sock, const char *mcast_cha
   apr_status_t status;
   int rval;
   apr_sockaddr_t *sa;
-
+ 
   status = apr_sockaddr_info_get(&sa, mcast_channel , APR_UNSPEC, port, 0, context);
   if(status != APR_SUCCESS)
     {
       return status;
     }
-
+ 
   switch( sa->family ) /* (*sa)->sa.sin.sin_family */
     {
-    case APR_INET:
-      {
+      case APR_INET:
+        {
           struct ifreq ifreq[1];
-
-	  memset(&ifreq,0, sizeof(ifreq));
-	  if(ifname)
-	    {
+ 
+          memset(&ifreq, 0, sizeof(ifreq));
+          if(ifname)
+            {
               strncpy(ifreq->ifr_name, ifname, IFNAMSIZ);
               if(ioctl(sock->socketdes, SIOCGIFADDR, ifreq) == -1)
-		{
-		  return APR_EGENERAL;
-		}	      
-	    }
-	  else
-	    {
-	      /* wildcard address (let the kernel decide) */
-	      ((struct sockaddr_in *)&ifreq->ifr_addr)->sin_addr.s_addr = htonl(INADDR_ANY);
-	    }
+                   return APR_EGENERAL;
+            }
+          else
+            {
+              /* wildcard address (let the kernel decide) */
+              ((struct sockaddr_in *)&ifreq->ifr_addr)->sin_addr.s_addr = htonl(INADDR_ANY);
+            }
 
-	  
           rval = setsockopt(sock->socketdes, IPPROTO_IP, IP_MULTICAST_IF,
-			    &((struct sockaddr_in *)&ifreq->ifr_addr)->sin_addr,
-	 		    sizeof( struct in_addr));
+                            &((struct sockaddr_in *)&ifreq->ifr_addr)->sin_addr,
+                            sizeof( struct in_addr));
 
-	  if(rval<0)
-	    {
-	      return APR_EGENERAL;
-	    }
-	  
-	  break;
-      }
+         if(rval<0)
+           {
+             return APR_EGENERAL;
+           }
 
-#if APR_HAVE_IPV6
-    case APR_INET6:
-      {
-	  u_int if_index = 0; /* Default interface */
-
-	  if(ifname)
-	    {
-	      if_index = if_nametoindex( ifname);
-	    }
-
-	  rval = setsockopt(sock->socketdes, IPPROTO_IPV6, IPV6_MULTICAST_IF,
-			    &if_index, sizeof(if_index));
-
-	  break;
-      }
+          break;
+        }
+ #if APR_HAVE_IPV6
+      case APR_INET6:
+        {
+          u_int if_index = 0; /* Default interface */
+ 
+          if(ifname)
+            {
+              if_index = if_nametoindex( ifname);
+            }
+ 
+          rval = setsockopt(sock->socketdes, IPPROTO_IPV6, IPV6_MULTICAST_IF,
+                           &if_index, sizeof(if_index));
+ 
+          break;
+        }
 #endif
-    default:
-      /* Set errno to EPROTONOSUPPORT */
-      return -1;
+      default:
+        /* Set errno to EPROTONOSUPPORT */
+        return -1;
     }
 
   return APR_SUCCESS;
 }
 
-
 static apr_status_t
 mcast_join( apr_pool_t *context, apr_socket_t *sock, char *mcast_channel, apr_port_t port, char *ifname )
 {
   apr_status_t status;
   int rval;
   apr_sockaddr_t *sa;
+  apr_os_sock_t s;
 
   status = apr_sockaddr_info_get(&sa, mcast_channel , APR_UNSPEC, port, 0, context);
   if(status != APR_SUCCESS)
@@ -317,71 +301,73 @@ mcast_join( apr_pool_t *context, apr_socket_t *sock, char *mcast_channel, apr_po
       return status;
     }
 
+  apr_os_sock_get(&s, sock);
+
   switch( sa->family ) /* (*sa)->sa.sin.sin_family */
     {
     case APR_INET:
-	{
-	  struct ip_mreq mreq[1];
-	  struct ifreq ifreq[1];
-
-	  /* &((*sa)->sa.sin.sin_addr */
-	  memcpy(&mreq->imr_multiaddr, &(sa->sa.sin.sin_addr), 
-		 sizeof mreq->imr_multiaddr);
-
-	  memset(&ifreq,0, sizeof(ifreq));
-	  if(ifname)
-	    {
-              memset(ifreq, 0, sizeof(struct ifreq));
-              strncpy(ifreq->ifr_name, ifname, IFNAMSIZ);
-              if(ioctl(sock->socketdes, SIOCGIFADDR, ifreq) == -1)
-		{
-		  return APR_EGENERAL;
-		}
-	    }
-	  else
-	    {
-	      /* wildcard address (let the kernel decide) */
-	      mreq->imr_interface.s_addr = htonl(INADDR_ANY);
-	    }
-
-	  memcpy(&mreq->imr_interface, 
-		 &((struct sockaddr_in *)&ifreq->ifr_addr)->sin_addr, 
-		 sizeof mreq->imr_interface);
-
-	  rval = setsockopt(sock->socketdes, IPPROTO_IP, IP_ADD_MEMBERSHIP, 
-			    mreq, sizeof mreq);
-	  if(rval<0)
-	    {
-	      return APR_EGENERAL;
-	    }
-	  break;
-	}
+      {
+        struct ip_mreq mreq[1];
+        struct ifreq ifreq[1];
+        
+        /* &((*sa)->sa.sin.sin_addr */
+        memcpy(&mreq->imr_multiaddr, &(sa->sa.sin.sin_addr), 
+               sizeof mreq->imr_multiaddr);
+        
+        memset(&ifreq,0, sizeof(ifreq));
+        if(ifname)
+          {
+            memset(ifreq, 0, sizeof(struct ifreq));
+            strncpy(ifreq->ifr_name, ifname, IFNAMSIZ);
+            if(ioctl(s, SIOCGIFADDR, ifreq) == -1)
+              {
+                return APR_EGENERAL;
+              }
+          }
+        else
+          {
+            /* wildcard address (let the kernel decide) */
+            mreq->imr_interface.s_addr = htonl(INADDR_ANY);
+          }
+        
+        memcpy(&mreq->imr_interface, 
+               &((struct sockaddr_in *)&ifreq->ifr_addr)->sin_addr, 
+               sizeof mreq->imr_interface);
+        
+        rval = setsockopt(s, IPPROTO_IP, IP_ADD_MEMBERSHIP, 
+                mreq, sizeof mreq);
+        if(rval<0)
+          {
+            return APR_EGENERAL;
+          }
+        break;
+      }
 #if APR_HAVE_IPV6
     case APR_INET6:
-	{
-	  struct ipv6_mreq mreq[1];
-	  struct ifreq ifreq[1];
-
-	  /* &((*sa)->sa.sin6.sin6_addr)*/
-          memcpy(&mreq->ipv6mr_multiaddr, &(sa->sa.sin6.sin6_addr),
-		                  sizeof mreq->ipv6mr_multiaddr);
-
-          memset(&ifreq,0, sizeof(ifreq));
-	  if(ifname)
-	    {
-	      strncpy(ifreq->ifr_name, ifname, IFNAMSIZ);
-	    }
-
-	  if (ioctl(sock->socketdes, SIOCGIFADDR, ifreq) == -1)
-	                return -1;
-
-	  rval = setsockopt(sock->socketdes, IPPROTO_IPV6, IPV6_JOIN_GROUP, mreq, sizeof mreq);
-	  break;
-	}
+      {
+        struct ipv6_mreq mreq[1];
+        struct ifreq ifreq[1];
+        
+        /* &((*sa)->sa.sin6.sin6_addr)*/
+        memcpy(&mreq->ipv6mr_multiaddr, &(sa->sa.sin6.sin6_addr),
+               sizeof mreq->ipv6mr_multiaddr);
+        
+        memset(&ifreq,0, sizeof(ifreq));
+        if(ifname)
+          {
+            strncpy(ifreq->ifr_name, ifname, IFNAMSIZ);
+          }
+        
+        if (ioctl(s, SIOCGIFADDR, ifreq) == -1)
+            return -1;
+        
+        rval = setsockopt(s, IPPROTO_IPV6, IPV6_JOIN_GROUP, mreq, sizeof mreq);
+        break;
+      }
 #endif
     default:
-      /* Set errno to EPROTONOSUPPORT */
-      return -1;
+        /* Set errno to EPROTONOSUPPORT */
+        return -1;
     }
 
   return APR_SUCCESS;
@@ -390,16 +376,16 @@ mcast_join( apr_pool_t *context, apr_socket_t *sock, char *mcast_channel, apr_po
 apr_socket_t *
 create_mcast_client(apr_pool_t *context, char *mcast_ip, apr_port_t port, int ttl, const char *interface)
 {
-  apr_socket_t *socket = create_udp_client(context, mcast_ip, port);
-  if(!socket)
-    {
-      return NULL;
-    }
-  mcast_set_ttl(socket, ttl);
+    apr_socket_t *socket = create_udp_client(context, mcast_ip, port);
+    if(!socket)
+      {
+        return NULL;
+      }
+    apr_mcast_hops(socket, ttl);
 
-  mcast_emit_on_if( context, socket, mcast_ip, port, interface);
+    mcast_emit_on_if( context, socket, mcast_ip, port, interface);
 
-  return socket;
+    return socket;
 }
 
 apr_socket_t *
