@@ -1,9 +1,9 @@
-/* $Id: hash.c 386 2004-11-18 01:49:29Z massie $ */
+/* $Id: hash.c 1146 2008-03-28 16:40:27Z bnicholes $ */
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
 #include "hash.h"
-#include "debug_msg.h"
+#include "ganglia.h"
 
 datum_t *
 datum_new ( void *data, size_t size )
