@@ -35,6 +35,9 @@
 #include <odmi.h>
 #include <cf.h>
 #include <sys/utsname.h>
+#include <sys/proc.h>
+#include <sys/types.h>
+#include <time.h>
 
 #include <libperfstat.h>
 
@@ -67,14 +70,22 @@ struct product {
         char *supersedes;        /* [512] offset: 0xf4 ( 244) */
 };
 
-
+#if defined(_AIX43)
+#ifndef SBITS
+/*
+ * For multiplication of fractions that are stored as integers, including
+ * p_pctcpu.  Not allowed to do floating point arithmetic in the kernel.
+ */
+#define SBITS   16
+#endif
+#endif
 
 #define MAX_CPUS  64
 
 #define INFO_TIMEOUT   10
 #define CPU_INFO_TIMEOUT INFO_TIMEOUT
 
-#define MEM_PAGESIZE 4096/1024
+#define MEM_KB_PER_PAGE (4096/1024)
 
 
 struct cpu_info {
@@ -102,8 +113,6 @@ int ci_flag=0;
 int ni_flag=0;
 
 perfstat_cpu_total_t cpu_total_buffer;
-perfstat_memory_total_t minfo;
-perfstat_disk_total_t dinfo;
 perfstat_netinterface_total_t ninfo[2],*last_ninfo, *cur_ninfo ;
 
 
@@ -115,7 +124,7 @@ struct cpu_info cpu_info[2],
   
 
 int aixver, aixrel, aixlev, aixfix;
-struct utsname unames;
+static time_t boottime;
 
 
 /* Prototypes
@@ -144,15 +153,11 @@ metric_init(void)
    cur_cpu_info->total_ticks = 0;
    
    update_ifdata();
-   uname(&unames);
    
    get_cpuinfo();
    sleep(CPU_INFO_TIMEOUT+1);
    get_cpuinfo();
 
-   perfstat_memory_total(NULL, &minfo, sizeof(perfstat_memory_total_t), 1);  
-   perfstat_disk_total(NULL, &dinfo, sizeof(perfstat_disk_total_t), 1);
-
    update_ifdata();
 
    bos_level(&aixver, &aixrel, &aixlev, &aixfix);
@@ -161,40 +166,45 @@ metric_init(void)
    return val;
 }
 
-
-
 g_val_t
 cpu_speed_func ( void )
 {
    g_val_t val;
+   perfstat_cpu_total_t c;
 
-   perfstat_cpu_total(NULL,  &cpu_total_buffer, sizeof(perfstat_cpu_total_t), 1);
-   val.uint32 = cpu_total_buffer.processorHZ/1000000;
+   if (perfstat_cpu_total(NULL,  &c, sizeof(perfstat_cpu_total_t), 1) == -1)
+      val.uint32 = 0;
+   else
+      val.uint32 = c.processorHZ/1000000;
 
    return val;
 }
 
-
 g_val_t
 boottime_func ( void )
 {
    g_val_t val;
-   int boottime = 0;
    struct utmp buf;
-   FILE *utmp = fopen(UTMP_FILE, "r");
+   FILE *utmp;
+   
+   if (!boottime) {
+      utmp = fopen(UTMP_FILE, "r");
 
-   if (utmp == NULL) {
-     /* Can't open utmp, use current time as boottime */
-     boottime = time(NULL);
-   } else {
-     while (fread((char *) &buf, sizeof(buf), 1, utmp) == 1) {
-        if (buf.ut_type == BOOT_TIME) {
-            boottime = buf.ut_time;
-            break;
-        }
-     }
+      if (utmp == NULL) {
+         /* Can't open utmp, use current time as boottime */
+         boottime = time(NULL);
+      } else {
+         while (fread((char *) &buf, sizeof(buf), 1, utmp) == 1) {
+            if (buf.ut_type == BOOT_TIME) {
+               boottime = buf.ut_time;
+               break;
+            }
+         }
+	 fclose (utmp);
+      }
    }
    val.uint32 = boottime;
+
    return val;
 }
 
@@ -207,24 +217,29 @@ sys_clock_func ( void )
    return val;
 }
 
-
-
 g_val_t
 machine_type_func ( void )
 {
    g_val_t val;
+   perfstat_cpu_total_t c;
 
+   if (perfstat_cpu_total (NULL, &c, sizeof( perfstat_cpu_total_t), 1) == -1)
+      strcpy (val.str, "unknown");
+   else
+      strncpy( val.str, c.description, MAX_G_STRING_SIZE );
 
-   strncpy( val.str,cpu_total_buffer.description , MAX_G_STRING_SIZE );
    return val;
 }
 
-
 g_val_t
 os_name_func ( void )
 {
    g_val_t val;
-   strncpy( val.str, unames.sysname, MAX_G_STRING_SIZE );
+   struct utsname uts;
+
+   uname (&uts);
+   strncpy (val.str, uts.sysname, MAX_G_STRING_SIZE);
+
    return val;
 }        
 
@@ -397,30 +412,35 @@ pkts_out_func ( void )
    return val;
 }
 
-
 g_val_t 
 disk_free_func ( void )
 {
    g_val_t val;
+   perfstat_disk_total_t d;
    
-   perfstat_disk_total(NULL, &dinfo, sizeof(perfstat_disk_total_t), 1);
-   val.d = dinfo.free;
+   if (perfstat_disk_total(NULL, &d, sizeof(perfstat_disk_total_t), 1) == -1)
+      val.d = 0.0;
+   else
+      val.d = (double)d.free / 1024;
+
    return val;
 }
 
-
 g_val_t 
 disk_total_func ( void )
 {
    g_val_t val;
-   perfstat_disk_total(NULL, &dinfo, sizeof(perfstat_disk_total_t), 1);
-   val.d =dinfo.size ;
+   perfstat_disk_total_t d;
+
+   if (perfstat_disk_total(NULL, &d, sizeof(perfstat_disk_total_t), 1) == -1)
+      val.d = 0.0;
+   else
+      val.d = (double)d.size / 1024;
+
    return val;
 }
 
-/* most used Partition dose not make sense to me 
-** FIXME
-*/
+/* FIXME */
 g_val_t 
 part_max_used_func ( void )
 {
@@ -429,15 +449,16 @@ part_max_used_func ( void )
    return val;
 }
 
-
-
 g_val_t
 load_one_func ( void )
 {
    g_val_t val;
+   perfstat_cpu_total_t c;
    
-   perfstat_cpu_total(NULL,  &cpu_total_buffer, sizeof(perfstat_cpu_total_t), 1);
-   val.f =(1.0*cpu_total_buffer.loadavg[0])/(1<<SBITS) ;
+   if (perfstat_cpu_total(NULL,  &c, sizeof(perfstat_cpu_total_t), 1) == -1)
+      val.f = 0.0;
+   else
+      val.f = (float)c.loadavg[0]/(float)(1<<SBITS);
    return val;
 }
 
@@ -445,9 +466,12 @@ g_val_t
 load_five_func ( void )
 {  
    g_val_t val;
+   perfstat_cpu_total_t c;
 
-   perfstat_cpu_total(NULL,  &cpu_total_buffer, sizeof(perfstat_cpu_total_t), 1);
-   val.f =(1.0*cpu_total_buffer.loadavg[1])/(1<<SBITS) ;
+   if (perfstat_cpu_total(NULL,  &c, sizeof(perfstat_cpu_total_t), 1) == -1)
+      val.f = 0.0;
+   else
+      val.f = (float)c.loadavg[1]/(float)(1<<SBITS);
    return val;
 }
 
@@ -455,9 +479,12 @@ g_val_t
 load_fifteen_func ( void )
 {
    g_val_t val;
+   perfstat_cpu_total_t c;
 
-   perfstat_cpu_total(NULL,  &cpu_total_buffer, sizeof(perfstat_cpu_total_t), 1);
-   val.f =(1.0*cpu_total_buffer.loadavg[2])/(1<<SBITS)*1.0 ;
+   if (perfstat_cpu_total(NULL, &c, sizeof(perfstat_cpu_total_t), 1) == -1)
+      val.f = 0.0;
+   else
+      val.f = (float)c.loadavg[2]/(float)(1<<SBITS);
 
    return val;
 }
@@ -466,9 +493,13 @@ g_val_t
 cpu_num_func ( void )
 {
    g_val_t val;
+   perfstat_cpu_total_t c;
+
+   if (perfstat_cpu_total(NULL,  &c, sizeof(perfstat_cpu_total_t), 1) == -1)
+      val.uint16 = 0;
+   else
+      val.uint16 = c.ncpus;
 
-   perfstat_cpu_total(NULL,  &cpu_total_buffer, sizeof(perfstat_cpu_total_t), 1);
-   val.uint16 =  cpu_total_buffer.ncpus;
    return val;
 }
 
@@ -585,21 +616,20 @@ proc_run_func( void )
   g_val_t val;
 
   val.uint32 = count_procs(1);
-
  
   return val;
 }
 
-
-
 g_val_t
 mem_total_func ( void )
 {
    g_val_t val;
+   perfstat_memory_total_t m;
   
-   perfstat_memory_total(NULL, &minfo, sizeof(perfstat_memory_total_t), 1);
-
-   val.uint32 = minfo.real_total*MEM_PAGESIZE;
+   if (perfstat_memory_total(NULL, &m, sizeof(perfstat_memory_total_t), 1) == -1)
+      val.f = 0;
+   else
+      val.f = m.real_total * MEM_KB_PER_PAGE;
    
    return val;
 }
@@ -608,35 +638,35 @@ g_val_t
 mem_free_func ( void )
 {
    g_val_t val;
+   perfstat_memory_total_t m;
   
-   perfstat_memory_total(NULL, &minfo, sizeof(perfstat_memory_total_t), 1);
+   if (perfstat_memory_total(NULL, &m, sizeof(perfstat_memory_total_t), 1) == -1)
+      val.f = 0;
+   else
+      val.f = m.real_free * MEM_KB_PER_PAGE;
    
-   val.uint32 = minfo.real_free*MEM_PAGESIZE; 
    return val;
 }
 
-/*
-** AIX does not have this
-** FIXME --
-*/
+/* FIXME? */
 g_val_t
 mem_shared_func ( void )
 {
    g_val_t val;
-   val.uint32 = 0;
+   
+   val.f = 0;
 
    return val;
 }
 
-/*
-** AIX does not have this
-** FIXME --
-*/
+/* FIXME? */
 g_val_t
 mem_buffers_func ( void )
 {
    g_val_t val;
-   val.uint32 = 0;
+   
+   val.f = 0;
+
    return val;
 }
 
@@ -644,10 +674,13 @@ g_val_t
 mem_cached_func ( void )
 {
    g_val_t val;
+   perfstat_memory_total_t m;
+
+   if (perfstat_memory_total(NULL, &m, sizeof(perfstat_memory_total_t), 1) == -1)
+      val.f = 0;
+   else
+      val.f = m.numperm * MEM_KB_PER_PAGE;
 
-   perfstat_memory_total(NULL, &minfo, sizeof(perfstat_memory_total_t), 1);
-   
-   val.uint32 = minfo.numperm*MEM_PAGESIZE;
    return val;
 }
 
@@ -655,10 +688,13 @@ g_val_t
 swap_total_func ( void )
 {
    g_val_t val;
+   perfstat_memory_total_t m;
 
-   perfstat_memory_total(NULL, &minfo, sizeof(perfstat_memory_total_t), 1);
+   if (perfstat_memory_total(NULL, &m, sizeof(perfstat_memory_total_t), 1) == -1)
+      val.f = 0;
+   else
+      val.f = m.pgsp_total * MEM_KB_PER_PAGE;
    
-   val.uint32 =minfo.pgsp_total*MEM_PAGESIZE;
    return val;
    
 }
@@ -667,14 +703,16 @@ g_val_t
 swap_free_func ( void )
 {
    g_val_t val;
-   perfstat_memory_total(NULL, &minfo, sizeof(perfstat_memory_total_t), 1);
-   
-   val.uint32 =minfo.pgsp_free*MEM_PAGESIZE;
+   perfstat_memory_total_t m;
+
+   if (perfstat_memory_total(NULL, &m, sizeof(perfstat_memory_total_t), 1) == -1)
+      val.f = 0;
+   else
+      val.f =m.pgsp_free * MEM_KB_PER_PAGE;
 
    return val;
 }
 
-
 g_val_t
 mtu_func ( void )
 {
