@@ -5,12 +5,18 @@
  *  Improved by Brooks Davis <brooks@one-eyed-alien.net>,
  *  Fixed libkvm code.
  *  Tue Jul 15 16:42:22 EST 2003
+ *  All bugs added by Carlo Marcelo Arenas Belon <carenas@sajinet.com.pe>
+ *
+ * Tested on FreeBSD 7 (amd64)
+ * Tested on FreeBSD 6.2 (amd64 and i386)
+ * Tested on FreeBSD 5.5 (i386)
  *
- * $Id: metrics.c 645 2006-04-14 20:16:06Z brooks_en_davis $
  */
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <ctype.h>
 
 #include <kvm.h>
 
@@ -31,6 +37,7 @@
 #include <sys/resource.h>
 #endif
 #include <sys/stat.h>
+#include <sys/vmmeter.h>
 #include <vm/vm_param.h>
 
 #include <sys/socket.h>
@@ -104,6 +111,7 @@ static int	  skipvfs;
 /* Function prototypes */
 static long percentages(int cnt, int *out, register long *new,
                           register long *old, long *diffs);
+int cpu_state(int which);
  
 /*
  * This function is called only once by the gmond.  Use to 
@@ -160,7 +168,8 @@ cpu_speed_func ( void )
    char buf[1024];
    char *curptr;
    size_t len;
-   long freq = 0, tmpfreq;
+   uint32_t freq = 0, tmpfreq;
+   uint64_t tscfreq;
 
    /*
     * If the system supports it, the cpufreq driver provides the best
@@ -179,24 +188,26 @@ cpu_speed_func ( void )
    while (isdigit(curptr[0])) {
       freq = strtol(curptr, &curptr, 10);
       if (freq > tmpfreq)
-	 tmpfreq = freq;
+         tmpfreq = freq;
       /* Skip the rest of this entry */
       while (!isspace(curptr[0]) && curptr[0] != '\0')
-	 curptr++;
+         curptr++;
       /* Find the next entry */
       while (!isdigit(curptr[0]) && curptr[0] != '\0')
-	 curptr++;
+         curptr++;
    }
    freq = tmpfreq;
-   if (tmpfreq != 0)
+   if (freq != 0)
       goto done;
 
    /*
     * machdep.tsc_freq exists on some i386/amd64 machines and gives the
     * CPU speed in Hz.  If it exists it's a decent value.
     */
-   if (sysctlbyname("machdep.tsc_freq", &tmpfreq, &len, NULL, 0) != -1) {
-      freq = (double)tmpfreq / 1e6;
+   tscfreq = 0;
+   len = sizeof(tscfreq);
+   if (sysctlbyname("machdep.tsc_freq", &tscfreq, &len, NULL, 0) != -1) {
+      freq = tscfreq / 1e6;
       goto done;
    }
 
@@ -211,13 +222,14 @@ mem_total_func ( void )
 {
    g_val_t val;
    size_t len;
-   long total;
+   u_long total;
 
    len = sizeof(total);
 
    if (sysctlbyname("hw.physmem", &total, &len, NULL, 0) == -1)
-	total = 0;
-   val.uint32 = total / 1024;
+      total = 0;
+
+   val.f = total / 1024;
 
    return val;
 }
@@ -230,7 +242,7 @@ swap_total_func ( void )
    struct xswdev xsw;
    size_t size;
    int totswap, n;
-   val.uint32 = 0;
+   val.f = 0;
    totswap = 0;
 
    if (use_vm_swap_info) {
@@ -246,12 +258,12 @@ swap_total_func ( void )
    } else if(kd != NULL) {
       n = kvm_getswapinfo(kd, swap, 1, 0);
       if (n < 0 || swap[0].ksw_total == 0) {
-         val.uint32 = 0;
+         val.f = 0;
       }
       totswap = swap[0].ksw_total;
     }
    
-   val.uint32 = totswap * (pagesize / 1024);
+   val.f = totswap * (pagesize / 1024);
    return val;
 }
 
@@ -332,7 +344,6 @@ int cpu_state(int which) {
    static struct timeval this_time, last_time;
    struct timeval time_diff;
    size_t len = sizeof(cp_time);
-   int i;
 
    if (which == -1) {
       bzero(cp_old, sizeof(cp_old));
@@ -484,16 +495,19 @@ g_val_t
 proc_total_func ( void )
 {
    g_val_t val;
-   size_t len = 0;
+   struct vmtotal total;
+   size_t len;
 
-   sysctlbyname("kern.proc.all", NULL, &len, NULL, 0);
+   /* computed every 5 seconds */
+   len = sizeof(total);
+   sysctlbyname("vm.vmtotal", &total, &len, NULL, 0);
 
-   val.uint32 = (len / sizeof (struct kinfo_proc)); 
+   val.uint32 = total.t_rq + \
+      total.t_dw + total.t_pw + total.t_sl + total.t_sw; 
 
    return val;
 }
 
-
 g_val_t
 proc_run_func( void )
 {
@@ -521,15 +535,15 @@ proc_run_func( void )
       state = kp->kp_proc.p_stat;
 #endif
       switch(state) {
-	 case SRUN:
-	 case SIDL:
-		val.uint32++;
-		break;
+         case SRUN:
+         case SIDL:
+            val.uint32++;
+            break;
       }
    }
 
    if (val.uint32 > 0)
-	val.uint32--;
+      val.uint32--;
 
 output:
    return val;
@@ -550,9 +564,9 @@ mem_free_func ( void )
 
    len = sizeof (free_pages);
    if((sysctlbyname("vm.stats.vm.v_free_count", &free_pages, &len, NULL, 0) 
-	== -1) || !len) free_pages = 0; 
+      == -1) || !len) free_pages = 0; 
 
-   val.uint32 = free_pages * (pagesize / 1024);
+   val.f = free_pages * (pagesize / 1024);
    return val;
 }
 
@@ -566,7 +580,7 @@ mem_shared_func ( void )
 {
    g_val_t val;
 
-   val.uint32 = 0;
+   val.f = 0;
 
    return val;
 }
@@ -584,10 +598,11 @@ mem_buffers_func ( void )
 
    len = sizeof (buffers);
    if((sysctlbyname("vfs.bufspace", &buffers, &len, NULL, 0) == -1) || !len)
-		buffers = 0; 
+      buffers = 0; 
+
    buffers /= 1024;
+   val.f = buffers;
 
-   val.uint32 = buffers;
    return val;
 }
 
@@ -604,10 +619,10 @@ mem_cached_func ( void )
 
    len = sizeof (cache);
    if((sysctlbyname("vm.stats.vm.v_cache_count", &cache, &len, NULL, 0) == -1) 
-	|| !len)
+      || !len)
       cache = 0; 
 
-   val.uint32 = cache * (pagesize / 1024);
+   val.f = cache * (pagesize / 1024);
    return val;
 }
 
@@ -620,7 +635,7 @@ swap_free_func ( void )
    struct xswdev xsw;
    size_t size;
    int totswap, usedswap, freeswap, n;
-   val.uint32 = 0;
+   val.f = 0;
    totswap = 0;
    usedswap = 0;
    if (use_vm_swap_info) {
@@ -640,7 +655,7 @@ swap_free_func ( void )
       usedswap = swap[0].ksw_used;
    }
    freeswap = totswap - usedswap;
-   val.uint32 = freeswap * (pagesize / 1024);
+   val.f = freeswap * (pagesize / 1024);
    return val;
 }
 
@@ -844,6 +859,7 @@ part_max_used_func( void )
  * at ftp://ftp.cs.berkeley.edu/pub/4bsd/README.Impt.License.Change.
  */
 
+/* FIXME: doesn't filter out devfs, bad reporting in amd64 */
 static float
 find_disk_space(double *total, double *tot_avail)
 {
@@ -1252,14 +1268,14 @@ counterdiff(uint64_t oldval, uint64_t newval, uint64_t maxval, uint64_t maxdiff)
 		return (newval - oldval);
 
 	/*
-         * Now the tricky part.  If we assume counters never get reset,
-         * this is easy.  Unfortunaly, they do get reset on some
-         * systems, so we need to try and deal with that.  Our huristic
-         * is that if out difference is greater then maxdiff and newval
-         * is less or equal to maxdiff, then we've probably been reset
-         * rather then actually wrapping.  Obviously, you need to be
-         * careful to poll often enough that you won't exceed maxdiff or
-         * you will get undersized numbers when you do wrap.
+	 * Now the tricky part.  If we assume counters never get reset,
+	 * this is easy.  Unfortunaly, they do get reset on some
+	 * systems, so we need to try and deal with that.  Our huristic
+	 * is that if out difference is greater then maxdiff and newval
+	 * is less or equal to maxdiff, then we've probably been reset
+	 * rather then actually wrapping.  Obviously, you need to be
+	 * careful to poll often enough that you won't exceed maxdiff or
+	 * you will get undersized numbers when you do wrap.
 	 */
 	diff = maxval - oldval + newval;
 	if (diff > maxdiff && newval <= maxdiff)
