@@ -1,5 +1,5 @@
 /*
- *  First stab at support for metrics in NetBSD
+ *  First stab at support for metrics in OpenBSD
  *  by Carlo Marcelo Arenas Belon <carenas@sajinet.com.pe>
  *  based on the NetBSD metrics by Jan Schaumann <jschauma@netbsd.org>
  *
@@ -32,6 +32,7 @@
 #include <limits.h>
 #include <paths.h>
 #include <stdlib.h>
+#include <string.h>
 
 #include "interface.h"
 #include "libmetrics.h"
@@ -72,9 +73,6 @@ static float	find_disk_space(double *, double *);
 // XXX
 //static int	vm_state(int);
 
-static int 	use_vm_swap_info = 0;
-static int 	mibswap[MIB_SWAPINFO_SIZE];
-static size_t 	mibswap_size;
 static kvm_t 	*kd = NULL;
 static int 	pagesize;
 static int	skipvfs;
@@ -128,7 +126,7 @@ cpu_speed_func ( void )
    mib[0] = CTL_HW;
    mib[1] = HW_CPUSPEED;
    if (sysctl(mib, 2, &cpu_speed, &len, NULL, 0) == -1 || !len)
-   	cpu_speed = 0;
+      cpu_speed = 0;
 
    val.uint32 = cpu_speed;
    return val;
@@ -146,7 +144,7 @@ mem_total_func ( void )
    mib[1] = HW_PHYSMEM;
    sysctl(mib, 2, &total, &len, NULL, 0);
    total /= 1024;
-   val.uint32 = total;
+   val.f = total;
    return val;
 }
 
@@ -159,7 +157,7 @@ swap_total_func ( void )
    int mib[2];
    struct uvmexp uvmexp;
 
-   val.uint32 = 0;
+   val.f = 0;
    totswap = 0;
 
    mib[0] = CTL_VM;
@@ -168,11 +166,11 @@ swap_total_func ( void )
    len = sizeof(uvmexp);
 
    if((sysctl(mib, 2, &uvmexp, &len, NULL, 0) < 0) || !len)
-   	totswap = 0;
+      totswap = 0;
    else
-	totswap = uvmexp.swpages;
+      totswap = uvmexp.swpages;
 
-   val.uint32 = totswap * (pagesize / 1024);
+   val.f = totswap * (pagesize / 1024);
 
    return val;
 }
@@ -487,9 +485,9 @@ mem_free_func ( void )
 	mib[0] = CTL_VM;
 	mib[1] = VM_METER;
 	if (sysctl(mib, 2, &vminfo, &len, NULL, 0) < 0)
-		val.uint32 = 0;
+		val.f = 0;
 	else
-		val.uint32 = vminfo.t_free * (pagesize / 1024);
+		val.f = vminfo.t_free * (pagesize / 1024);
 	return val;
 }
 
@@ -505,9 +503,9 @@ mem_shared_func ( void )
 	mib[0] = CTL_VM;
 	mib[1] = VM_METER;
 	if (sysctl(mib, 2, &vminfo, &len, NULL, 0) < 0)
-		val.uint32 = 0;
+		val.f = 0;
 	else
-		val.uint32 = (vminfo.t_vmshr + vminfo.t_rmshr) * (pagesize / 1024);
+		val.f = (vminfo.t_vmshr + vminfo.t_rmshr) * (pagesize / 1024);
 	return val;
 }
 
@@ -524,10 +522,11 @@ mem_buffers_func ( void )
 
    len = sizeof (buffers);
    if((sysctl(mib, 2, &buffers, &len, NULL, 0) == -1) || !len)
-		buffers = 0; 
+      buffers = 0; 
+
    buffers /= 1024;
+   val.f = buffers;
 
-   val.uint32 = buffers;
    return val;
 }
 
@@ -550,7 +549,7 @@ mem_cached_func ( void )
    else
       cache = uvmexp.vnodepages + uvmexp.vtextpages;
 
-   val.uint32 = cache * (pagesize / 1024);
+   val.f = cache * (pagesize / 1024);
    return val;
 }
 
@@ -574,7 +573,7 @@ swap_free_func ( void )
    else
       totswapf = uvmexp.swpages - uvmexp.swpginuse;
 
-   val.uint32 = totswapf * (pagesize / 1024);
+   val.f = totswapf * (pagesize / 1024);
    return val;
 }
 
@@ -845,7 +844,6 @@ makenetvfslist(void)
 
 	int mib[3], maxvfsconf;
 	size_t miblen;
-	struct ovfsconf *ptr;
 
 	mib[0] = CTL_VFS; mib[1] = VFS_GENERIC; mib[2] = VFS_MAXTYPENUM;
 	miblen=sizeof(maxvfsconf);
@@ -861,16 +859,6 @@ makenetvfslist(void)
 	}
 
 	cnt = 0;
-	/* XXX: use sysctl ? */
-	/* while ((ptr = getvfsent()) != NULL && cnt < maxvfsconf) {
-
-		listptr[cnt] = strdup(ptr->vfc_name);
-		if (listptr[cnt] == NULL) {
-			warnx("malloc failed");
-			goto done;
-		}
-		cnt++;
-	} */
 	if (cnt == 0)
 		goto done;
 	/*
@@ -1085,14 +1073,14 @@ counterdiff(uint64_t oldval, uint64_t newval, uint64_t maxval, uint64_t maxdiff)
 		return (newval - oldval);
 
 	/*
-         * Now the tricky part.  If we assume counters never get reset,
-         * this is easy.  Unfortunaly, they do get reset on some
-         * systems, so we need to try and deal with that.  Our huristic
-         * is that if out difference is greater then maxdiff and newval
-         * is less or equal to maxdiff, then we've probably been reset
-         * rather then actually wrapping.  Obviously, you need to be
-         * careful to poll often enough that you won't exceed maxdiff or
-         * you will get undersized numbers when you do wrap.
+	 * Now the tricky part.  If we assume counters never get reset,
+	 * this is easy.  Unfortunaly, they do get reset on some
+	 * systems, so we need to try and deal with that.  Our huristic
+	 * is that if out difference is greater then maxdiff and newval
+	 * is less or equal to maxdiff, then we've probably been reset
+	 * rather then actually wrapping.  Obviously, you need to be
+	 * careful to poll often enough that you won't exceed maxdiff or
+	 * you will get undersized numbers when you do wrap.
 	 */
 	diff = maxval - oldval + newval;
 	if (diff > maxdiff && newval <= maxdiff)
