@@ -1,6 +1,5 @@
 #include "interface.h"
 #include "libmetrics.h"
-#include "debug_msg.h"
 
 /* This is file is an amended version of solaris.c included in ganglia 2.5.4 
  * and ganglia 2.5.5. It has been modified by Adeyemi Adesanya 
@@ -188,9 +187,9 @@ get_kstat_val(g_val_t *val, char *km_name, char *ks_name, char *name)
    ** Get a kstat_ctl handle, or update the kstat chain.
    */
    if (kc == NULL)
-	kc = kstat_open();
+      kc = kstat_open();
    else
-	kstat_chain_update(kc);
+      kstat_chain_update(kc);
 
    if (kc == NULL)
       {
@@ -304,12 +303,12 @@ update_metric_data ( void )
  * stuff to match.
  */
 
-long percentages(int cnt, int *out, register unsigned long *new, register unsigned long *old, long unsigned *diffs)
+long percentages(int cnt, int *out, register unsigned long *new, register unsigned long *old, unsigned long *diffs)
 {
     register int i;
     register long change;
     register long total_change;
-    register long *dp;
+    register unsigned long *dp;
     long half_total;
 
     /* initialization */
@@ -396,9 +395,9 @@ determine_cpu_percentages ( void )
 
    if (lasttime.tv_sec)
      timediff = ((double) thistime.tv_sec * 1.0e7 +
-		 ((double) thistime.tv_usec * 10.0)) -
-	((double) lasttime.tv_sec * 1.0e7 +
-	 ((double) lasttime.tv_usec * 10.0));
+                ((double) thistime.tv_usec * 10.0)) -
+                ((double) lasttime.tv_sec * 1.0e7 +
+                ((double) lasttime.tv_usec * 10.0));
    else
      timediff = 1.0e7;
 
@@ -495,12 +494,12 @@ determine_cpu_percentages ( void )
      
       /* sum up to the wait state counter, the last two we determine ourselves */
       for (j = 0; j < CPU_WAIT; j++){
-	   cpu_now[j] += (unsigned long) cpuKstats.cpu_sysinfo.cpu[j];
+         cpu_now[j] += (unsigned long) cpuKstats.cpu_sysinfo.cpu[j];
       }
       
 
       cpu_now[CPUSTATE_IOWAIT] += (unsigned long) cpuKstats.cpu_sysinfo.wait[W_IO] +
-				  (unsigned long) cpuKstats.cpu_sysinfo.wait[W_PIO];
+                                  (unsigned long) cpuKstats.cpu_sysinfo.wait[W_PIO];
       cpu_now[CPUSTATE_SWAP] += (unsigned long) cpuKstats.cpu_sysinfo.wait[W_SWAP];
       
       buffers[0].bread += (long)cpuKstats.cpu_sysinfo.bread;
@@ -538,16 +537,13 @@ determine_cpu_percentages ( void )
 /* decay stuff
  * semi-stolen from top.  :)  added by swagner on 8/20/02
  */
-   if (time_delta < 30)
-	{
-	alpha = 0.5 * (time_delta / 30);
-	beta = 1.0 - alpha;
-	}
-   else
-	{
-	alpha = 0.5;
-	beta = 0.5;
-	}
+   if (time_delta < 30) {
+      alpha = 0.5 * (time_delta / 30);
+      beta = 1.0 - alpha;
+   } else {
+      alpha = 0.5;
+      beta = 0.5;
+   }
    metriclist.bread_sec.f = (float)buffers[2].bread / (float)time_delta;
    if (buffers[1].bread == buffers[0].bread)
       metriclist.bread_sec.f = 0.;
@@ -576,7 +572,7 @@ determine_cpu_percentages ( void )
            metriclist.bread_sec.f, metriclist.bwrite_sec.f,
            metriclist.lread_sec.f, metriclist.lwrite_sec.f,
            metriclist.phread_sec.f, metriclist.phwrite_sec.f,
-	   time_delta
+           time_delta
    );
 
    buffers[1].bread = buffers[0].bread;
@@ -752,9 +748,9 @@ update_if_data(void)
    gettimeofday (&thistime, NULL);
    if (lasttime.tv_sec)
      timediff = ((double) thistime.tv_sec * 1.0e6 +
-		 (double) thistime.tv_usec -
-		 (double) lasttime.tv_sec * 1.0e6 -
-		 (double) lasttime.tv_usec) / 1.0e6;
+                 (double) thistime.tv_usec -
+                 (double) lasttime.tv_sec * 1.0e6 -
+                 (double) lasttime.tv_usec) / 1.0e6;
    else
      timediff = 1.0;
 
@@ -769,9 +765,9 @@ update_if_data(void)
    ** Get a kstat_ctl handle, or update the kstat chain.
    */
    if (kc == NULL)
-	kc = kstat_open();
+      kc = kstat_open();
    else
-	kstat_chain_update(kc);
+      kstat_chain_update(kc);
 
    if (kc == NULL)
       {
@@ -882,7 +878,7 @@ mem_total_func ( void )
 {
    g_val_t val;
 
-   val.uint32 = metriclist.mem_total.uint32;
+   val.f = metriclist.mem_total.uint32;
 //   val.uint32 = pagetok(sysconf(_SC_PHYS_PAGES));
    return val;
 }
@@ -892,7 +888,7 @@ swap_total_func ( void )
 {
    g_val_t val;
    metric_tick();
-   val.uint32 = metriclist.swap_total.uint32;
+   val.f = metriclist.swap_total.uint32;
    return val;
 }
 
@@ -952,6 +948,7 @@ cpu_user_func ( void )
    return val;
 }
 
+/* FIXME: ? */
 g_val_t
 cpu_nice_func ( void )
 {
@@ -979,6 +976,7 @@ cpu_idle_func ( void )
    return val;
 }
 
+/* FIXME: always 0? */
 g_val_t 
 cpu_wio_func ( void )
 {
@@ -1085,7 +1083,7 @@ mem_free_func ( void )
    g_val_t val;
 
    metric_tick();
-   val.uint32 = metriclist.mem_free.uint32;
+   val.f = metriclist.mem_free.uint32;
    return val;
 }
 
@@ -1098,7 +1096,7 @@ mem_shared_func ( void )
 
    g_val_t val;
 
-   val.uint32 = 0;
+   val.f = 0;
    return val;
 }
 
@@ -1111,7 +1109,7 @@ mem_buffers_func ( void )
 
    g_val_t val;
 
-   val.uint32 = 0;
+   val.f = 0;
    return val;
 }
 
@@ -1124,7 +1122,7 @@ mem_cached_func ( void )
 
    g_val_t val;
 
-   val.uint32 = 0;
+   val.f = 0;
    return val;
 }
 
@@ -1134,7 +1132,7 @@ swap_free_func ( void )
    g_val_t val;
 
    metric_tick();
-   val.uint32 = metriclist.swap_free.uint32;
+   val.f = metriclist.swap_free.uint32;
    return val;
 }
 
@@ -1275,7 +1273,6 @@ float find_disk_space(double *total_size, double *total_free)
 {
    FILE *mounts;
    struct mnttab mp;
-   struct statvfs buf;
    char *mount, *device, *type;
    /* We report in GB = 1 thousand million bytes */
    const double reported_units = 1e9;
@@ -1349,61 +1346,3 @@ part_max_used_func ( void )
    val.f = most_full;
    return val;
 }
-
-/* ---------------------------------- */
-/*
- * Stub for doing stand-alone testing
- *
- * Compile with:
- *
- *
-gcc -m64 -I.. -I../lib -I../lib/dnet -g -O2 -Wall -DHAVE_CONFIG_H -D__EXTENSIONS__ -D_POSIX_C_SOURCE=199506L \ 
-   -DHAVE_STRERROR -DSTAND_ALONE_TEST  -o gmo.tst machine.c ../lib/.libs/libganglia.a \ 
-   ../lib/libgetopthelper.a -ldl -lkstat -lresolv -lnsl -lsocket -lpthread
- *
- */
-#ifdef STAND_ALONE_TEST
-int main()
-{
-g_val_t tval;
-
-metric_init();
-
-tval = proc_total_func();
-printf("Tot proc: %d\n",tval.uint32);
-tval = proc_run_func();
-printf("Run proc: %d\n",tval.uint32);
-
-tval = pkts_in_func();
-printf("ipkts = %f\n",tval.f);
-tval = pkts_out_func();
-printf("opkts = %f\n",tval.f);
-tval = bytes_in_func();
-printf("ibytes = %f\n",tval.f);
-tval = bytes_out_func();
-printf("obytes = %f\n",tval.f);
-sleep(11);
-tval = pkts_in_func();
-printf("ipkts = %f\n",tval.f);
-tval = pkts_out_func();
-printf("opkts = %f\n",tval.f);
-tval = bytes_in_func();
-printf("ibytes = %f\n",tval.f);
-tval = bytes_out_func();
-printf("obytes = %f\n",tval.f);
-sleep(11);
-tval = pkts_in_func();
-printf("ipkts = %f\n",tval.f);
-tval = pkts_out_func();
-printf("opkts = %f\n",tval.f);
-tval = bytes_in_func();
-printf("ibytes = %f\n",tval.f);
-tval = bytes_out_func();
-printf("obytes = %f\n",tval.f);
-
-tval = cpu_aidle_func();
-printf("aidlepct = %f\n",tval.f);
-
-return 0;
-}
-#endif
