@@ -1,5 +1,5 @@
 <?php
-# $Id: conf.php 905 2007-12-11 22:38:21Z bernardli $
+# $Id: conf.php.in 1831 2008-09-26 12:18:54Z carenas $
 #
 # Gmetad-webfrontend version. Used to check for updates.
 #
@@ -25,6 +25,9 @@ $rrds = "$gmetad_root/rrds";
 # otherwise, change it if it is installed elsewhere (like /usr/bin)
 define("RRDTOOL", "/usr/bin/rrdtool");
 
+# Location for modular-graph files.
+$graphdir='./graph.d';
+
 #
 # If you want to grab data from a different ganglia source specify it here.
 # Although, it would be strange to alter the IP since the Round-Robin
@@ -33,10 +36,6 @@ define("RRDTOOL", "/usr/bin/rrdtool");
 $ganglia_ip = "127.0.0.1";
 $ganglia_port = 8652;
 
-# Old-style names.
-$gmetad_ip = $ganglia_ip;
-$gmetad_port = $ganglia_port;
-
 #
 # The maximum number of dynamic graphs to display.  If you set this
 # to 0 (the default) all graphs will be shown.  This option is
@@ -45,6 +44,18 @@ $gmetad_port = $ganglia_port;
 #
 $max_graphs = 0;
 
+#
+# In the Cluster View this sets the default number of columns used to
+# display the host grid below the summary graphs.
+#
+$hostcols = 4;
+
+#
+# In the Host View this sets the default number of columns used to
+# display the metric grid below the summary graphs.
+#
+$metriccols = 2;
+
 #
 # Turn on and off the Grid Snapshot. Now that we have a
 # hierarchical snapshot (per-cluster instead of per-node) on
@@ -96,7 +107,6 @@ $load_colors = array(
    "75-100" =>"ffa15e",
    "50-75" => "ffde5e",
    "25-50" => "caff98",
-   #"0-25" => "efefef",
    "0-25" => "e2ecff",
    "down" => "515151"
 );
@@ -111,16 +121,17 @@ $load_scale = 1.0;
 #
 $default_metric_color = "555555";
 
-#
-# Default graph range (hour, day, week, month, or year)
-#
-$default_range = "hour";
-
 #
 # Default metric 
 #
 $default_metric = "load_one";
 
+#
+# remove the domainname from the FQDN hostnames in graphs
+# (to help with long hostnames in small charts)
+#
+$strip_domainname = false;
+
 #
 # Optional summary graphs
 #
@@ -166,11 +177,7 @@ $graph_sizes = array(
      'fudge_1'=>0,
      'fudge_2'=>0
    ),
-   'xlarge'=>array(
-     'height'=>1200,
-     'width'=>1600,
-     'fudge_2'=>0
-   ),
+   # this was the default value when no other size was provided.
    'default'=>array(
      'height'=>100,
      'width'=>400,
@@ -179,9 +186,6 @@ $graph_sizes = array(
      'fudge_2'=>0
    )
 );
-
-# this key must exist in $graph_sizes
 $default_graph_size = 'default';
-
 $graph_sizes_keys = array_keys( $graph_sizes );
 ?>
