@@ -174,6 +174,8 @@ function load_color ($value)
 # the load/death of a cluster node
 function node_image ($metrics)
 {
+   global $hosts_down;
+
    $cpu_num  = $metrics["cpu_num"]['VAL'];
    if(!$cpu_num || $cpu_num == 0)
       {
@@ -223,7 +225,7 @@ function find_limits($nodes, $metricname)
 
          $rrd_dir = "$rrds/$clustername/$host";
          if (file_exists("$rrd_dir/$metricname.rrd")) {
-		$command = RRDTOOL . " graph - --start $start --end $end ".
+		$command = RRDTOOL . " graph '' --start $start --end $end ".
 		"DEF:limits='$rrd_dir/$metricname.rrd':'sum':AVERAGE ".
 		"PRINT:limits:MAX:%.2lf ".
 		"PRINT:limits:MIN:%.2lf";
@@ -365,10 +367,10 @@ function physical_racks()
    if (is_array($hosts_up)) {
       foreach ($hosts_up as $host=>$v) {
          # Try to find the node's location in the cluster.
-         list($rack, $rank) = findlocation($v);
+         list($rack, $rank, $plane) = findlocation($v);
 
-         if ($rack>=0 and $rank>=0) {
-            $racks[$rack][$rank]=$v['NAME'];
+         if ($rack>=0 and $rank>=0 and $plane>=0) {
+            $racks[$rack][]=$v['NAME'];
             continue;
          }
          else {
@@ -382,9 +384,9 @@ function physical_racks()
    }
    if (is_array($hosts_down)) {
       foreach ($hosts_down as $host=>$v) {
-         list($rack, $rank) = findlocation($v);
-         if ($rack>=0 and $rank>=0) {
-            $racks[$rack][$rank]=$v['NAME'];
+         list($rack, $rank, $plane) = findlocation($v);
+         if ($rack>=0 and $rank>=0 and $plane>=0) {
+            $racks[$rack][]=$v['NAME'];
             continue;
          }
          else {
@@ -419,6 +421,10 @@ function clean_string( $string )
 {
   return htmlentities( $string );
 }
+#-------------------------------------------------------------------------------
+function sanitize ( $string ) {
+  return  escapeshellcmd( clean_string( rawurldecode( $string ) ) ) ;
+}
 
 #-------------------------------------------------------------------------------
 # If arg is a valid number, return it.  Otherwise, return null.
@@ -440,4 +446,19 @@ function is_valid_hex_color( $string )
   return $return_value;
     
 }
+
+#-------------------------------------------------------------------------------
+# Return a shortened version of a FQDN
+# if "hostname" is numeric only, assume it is an IP instead
+# 
+function strip_domainname( $hostname ) {
+    $postition = strpos($hostname, '.');
+    $name = substr( $hostname , 0, $postition );
+    if ( FALSE === $postition || is_numeric($name) ) {
+        return $hostname;
+    } else {
+        return $name;
+    }
+}
+
 ?>
