@@ -1,6 +1,8 @@
 <?php
-/* $Id: get_context.php 970 2008-02-14 19:16:19Z carenas $ */
+/* $Id: get_context.php 1831 2008-09-26 12:18:54Z carenas $ */
 
+include_once "./functions.php";
+  
 $meta_designator = "Grid";
 $cluster_designator = "Cluster Overview";
 
@@ -15,12 +17,17 @@ $range = isset( $_GET["r"] ) && in_array($_GET["r"], array_keys( $time_ranges )
 	escapeshellcmd( rawurldecode($_GET["r"])) : NULL;
 $metricname = isset($_GET["m"]) ?
 	escapeshellcmd( clean_string( rawurldecode($_GET["m"]) ) ) : NULL;
+$metrictitle = isset($_GET["ti"]) ?
+	escapeshellcmd( clean_string( rawurldecode($_GET["ti"]) ) ) : NULL;
 $sort = isset($_GET["s"]) ?
 	escapeshellcmd( clean_string( rawurldecode($_GET["s"]) ) ) : NULL;
 $controlroom = isset($_GET["cr"]) ?
 	clean_number( rawurldecode($_GET["cr"]) ) : NULL;
-$hostcols = isset($_GET["hc"]) ?
-	clean_number( $_GET["hc"] ) : NULL;
+# Default value set in conf.php, Allow URL to overrride
+if (isset($_GET["hc"]))
+	$hostcols = clean_number($_GET["hc"]);
+if (isset($_GET["mc"]))
+	$metriccols = clean_number($_GET["mc"]);
 # Flag, whether or not to show a list of hosts
 $showhosts = isset($_GET["sh"]) ?
 	clean_number( $_GET["sh"] ) : NULL;
@@ -43,17 +50,23 @@ $clustergraphsize = isset($_GET["z"]) && in_array( $_GET[ 'z' ], $graph_sizes_ke
     escapeshellcmd($_GET["z"]) : NULL;
 # A stack of grid parents. Prefer a GET variable, default to cookie.
 if (isset($_GET["gs"]) and $_GET["gs"])
-      $gridstack = explode(":", clean_string( rawurldecode($_GET["gs"] ) ) );
-else
-      $gridstack = explode(":", clean_string( $_COOKIE["gs"] ) );
+    $gridstack = explode( ">", rawurldecode( $_GET["gs"] ) );
+else if ( isset($_COOKIE['gs']) and $_COOKIE['gs'])
+    $gridstack = explode( ">", $_COOKIE["gs"] );
 
-# Assume we are the first grid visited in the tree if there are no CGI variables,
+if (isset($gridstack) and $gridstack) {
+   foreach( $gridstack as $key=>$value )
+      $gridstack[ $key ] = clean_string( $value );
+}
+
+# Assume we are the first grid visited in the tree if there is no gridwalk
 # or gridstack is not well formed. Gridstack always has at least one element.
-if (!count($_GET) or !strstr($gridstack[0], "http://"))
-      $initgrid=TRUE;
+if ( !isset($gridstack) or !strstr($gridstack[0], "http://"))
+    $initgrid = TRUE;
 
 # Default values
-if (!is_numeric($hostcols)) $hostcols = 4;
+if (!isset($hostcols) || !is_numeric($hostcols)) $hostcols = 4;
+if (!isset($metriccols) || !is_numeric($metriccols)) $metriccols = 2;
 if (!is_numeric($showhosts)) $showhosts = 1;
 
 # Set context.
@@ -95,7 +108,7 @@ else if($clustername and $hostname)
    }
 
 if (!$range)
-      $range = "$default_range";
+      $range = "$default_time_range";
 
 $end = "N";
 
