@@ -1,5 +1,5 @@
 <?php
-/* $Id: host_view.php 589 2005-10-10 07:42:26Z knobi1 $ */
+/* $Id: host_view.php 1831 2008-09-26 12:18:54Z carenas $ */
 
 $tpl = new TemplatePower( template("host_view.tpl") );
 $tpl->assignInclude("extra", template("host_extra.tpl"));
@@ -31,14 +31,16 @@ if ($hosts_down)
    }
 
 $tpl->assign("ip", $hosts_up['IP']);
+$tpl->newBlock('columns_dropdown');
+$tpl->assign("metric_cols_menu", $metric_cols_menu);
+$g_metrics_group = array();
 
 foreach ($metrics as $name => $v)
    {
        if ($v['TYPE'] == "string" or $v['TYPE']=="timestamp" or
            (isset($always_timestamp[$name]) and $always_timestamp[$name]))
           {
-             # Long gmetric name/values will disrupt the display here.
-             if ($v['SOURCE'] == "gmond") $s_metrics[$name] = $v;
+             $s_metrics[$name] = $v;
           }
        elseif ($v['SLOPE'] == "zero" or
                (isset($always_constant[$name]) and $always_constant[$name]))
@@ -57,20 +59,43 @@ foreach ($metrics as $name => $v)
                 $encodeUnits = rawurlencode($v['UNITS']);
                 $graphargs .= "&vl=$encodeUnits";
              }
+             if (isset($v['TITLE'])) {
+                $title = $v['TITLE'];
+                $graphargs .= "&ti=$title";
+             }
              $g_metrics[$name]['graph'] = $graphargs;
+             $g_metrics[$name]['description'] = isset($v['DESC']) ? $v['DESC'] : '';
+
+             # Setup an array of groups that can be used for sorting in group view
+             if ( isset($metrics[$name]['GROUP']) ) {
+                $groups = $metrics[$name]['GROUP'];
+             } else {
+                $groups = array("");
+             }
+
+             foreach ( $groups as $group) {
+                if ( isset($g_metrics_group[$group]) ) {
+                   $g_metrics_group[$group] = array_merge($g_metrics_group[$group], (array)$name);
+                } else {
+                   $g_metrics_group[$group] = array($name);
+                }
+             }
           }
    }
 # Add the uptime metric for this host. Cannot be done in ganglia.php,
 # since it requires a fully-parsed XML tree. The classic contructor problem.
 $s_metrics['uptime']['TYPE'] = "string";
 $s_metrics['uptime']['VAL'] = uptime($cluster['LOCALTIME'] - $metrics['boottime']['VAL']);
+$s_metrics['uptime']['TITLE'] = "Uptime";
 
 # Add the gmond started timestamps & last reported time (in uptime format) from
 # the HOST tag:
 $s_metrics['gmond_started']['TYPE'] = "timestamp";
 $s_metrics['gmond_started']['VAL'] = $hosts_up['GMOND_STARTED'];
+$s_metrics['gmond_started']['TITLE'] = "Gmond Started";
 $s_metrics['last_reported']['TYPE'] = "string";
 $s_metrics['last_reported']['VAL'] = uptime($cluster['LOCALTIME'] - $hosts_up['REPORTED']);
+$s_metrics['last_reported']['TITLE'] = "Last Reported";
 
 # Show string metrics
 if (is_array($s_metrics))
@@ -81,7 +106,12 @@ if (is_array($s_metrics))
 	# RFM - If units aren't defined for metric, make it be the empty string
 	! array_key_exists('UNITS', $v) and $v['UNITS'] = "";
         $tpl->newBlock("string_metric_info");
-        $tpl->assign("name", $name);
+		if (isset($v['TITLE'])) {
+			$tpl->assign("name", $v['TITLE']);
+		}
+		else {
+			$tpl->assign("name", $name);
+		}
         if( $v['TYPE']=="timestamp" or (isset($always_timestamp[$name]) and $always_timestamp[$name]))
            {
               $tpl->assign("value", date("r", $v['VAL']));
@@ -100,25 +130,45 @@ if (is_array($c_metrics))
       foreach ($c_metrics as $name => $v )
      {
         $tpl->newBlock("const_metric_info");
-        $tpl->assign("name", $name);
+		if (isset($v['TITLE'])) {
+			$tpl->assign("name", $v['TITLE']);
+		}
+		else { 
+			$tpl->assign("name", $name);
+		}
         $tpl->assign("value", "$v[VAL] $v[UNITS]");
      }
    }
 
 # Show graphs.
-if (is_array($g_metrics))
+if ( is_array($g_metrics) && is_array($g_metrics_group) )
    {
-      ksort($g_metrics);
+      ksort($g_metrics_group);
 
-      $i = 0;
-      foreach ( $g_metrics as $name => $v )
+      foreach ( $g_metrics_group as $group => $metric_array )
          {
-            $tpl->newBlock("vol_metric_info");
-            $tpl->assign("graphargs", $v['graph']);
-            $tpl->assign("alt", "$hostname $name");
-            if($i++ %2)
-               $tpl->assign("br", "<BR>");
+            if ( $group == "" ) {
+               $group = "no_group";
+	    }
+            $tpl->newBlock("vol_group_info");
+            $tpl->assign("group", $group);
+            $tpl->assign("group_metric_count", count($metric_array));
+            $i = 0;
+            ksort($g_metrics);
+            foreach ( $g_metrics as $name => $v )
+               {
+                  if ( in_array($name, $metric_array) ) {
+                     $tpl->newBlock("vol_metric_info");
+                     $tpl->assign("graphargs", $v['graph']);
+                     $tpl->assign("alt", "$hostname $name");
+                     if (isset($v['description']))
+                       $tpl->assign("desc", $v['description']);
+                     if ( !(++$i % $metriccols) )
+                        $tpl->assign("br", "<BR>");
+                  }
+               }
          }
+
    }
 
 $tpl->printToScreen();
